#ifndef DRECONFIG_H
#define DRECONFIG_H

#include <QString>
#include <QVariant>
#include <QMetaType>
#include <optional>

#include "base/DeviceIndex.h"

enum REG_OP_ACTION_TYPE{
    REG_OP_ACTION_16=0,
    REG_OP_ACTION_32=1,
    REG_OP_ACTION_64=2
};

struct RegOpAction
{
    friend bool operator==(const RegOpAction &, const RegOpAction &) = default;
    std::optional<quint64> getWriteVal() const;

    quint32 sdbId = 0;
    quint32 sdbSeq = 0;
    quint16 regOffset = 0;
    REG_OP_ACTION_TYPE regType = REG_OP_ACTION_16;
    bool isWrite = false;
    QString writeVal = "0";
    QString comment;
};
Q_DECLARE_METATYPE(RegOpAction)

struct ConfigBase
{
    friend bool operator==(const ConfigBase &, const ConfigBase &) = default;

    enum REG_TYPE{REG_TYPE_16=0,
                  REG_TYPE_32=1,
                  REG_TYPE_64=2};

    DeviceIndex deviceIndex;
    bool isCustom = false;
    REG_TYPE regType = REG_TYPE_16;
    quint16 regAddr = 0;
    quint64 regWriteVal = 0;

    QList<RegOpAction> regOpActions;
};
Q_DECLARE_METATYPE(ConfigBase)

class DreConfig : public ConfigBase
{
public:
    friend bool operator==(const DreConfig &, const DreConfig &) = default;
    void loadSettings();
    void saveSettings();
};
Q_DECLARE_METATYPE(DreConfig)

#endif // DRECONFIG_H
