#ifndef DREMAINWINDOW_H
#define DREMAINWINDOW_H

#include <QThread>

#include "base/app/BaseMainWindow.h"
#include "DreConfig.h"

class DeviceDispatcher;
class DeviceIndexEnabledMap;
class DiscoverDialog;
class QtMregDevice;
class RegOpModel;

namespace Ui {
class DreMainWindow;
}


class DreMainWindow : public BaseMainWindow
{
    Q_OBJECT
public:
    explicit DreMainWindow(QWidget *parent = nullptr);
    ~DreMainWindow() override;

signals:
    void deviceListChanged(const DeviceIndexEnabledMap &);
    void regResultsChanged(const QVector<quint64> &results);

public slots:
    void setDeviceList(const DeviceIndexEnabledMap &devices);

private Q_SLOTS:
    void on_comboBoxRegAddr_currentIndexChanged(int);
    void on_lineEditRegAddr_textEdited(const QString);
    void on_comboBoxRegType_currentIndexChanged(int);
    void on_pushButtonRead_clicked();
    void on_pushButtonWrite_clicked();
    void on_pushButtonAdd_clicked();
    void on_pushButtonRemove_clicked();
    void on_pushButtonExec_clicked();

    void deviceConnected();
    void deviceDisconnected();
    void setConfig(const DreConfig &newConfig);

private:
    void addDevice(const DeviceIndex &index);
    void removeDevice(const DeviceIndex &index);
    QtMregDevice *createDevice(const DeviceIndex &index);
    bool saveConfig(bool checkWriteVal=true);
    void restoreGui();
    void updateLineEditWriteLen();
    void onDeviceStateChange();
    void updateSdbComboBox();

    Ui::DreMainWindow *ui;
    DreConfig config;
    QSharedPointer<QtMregDevice> dev;
    bool connectedState = false;
    DeviceDispatcher * const deviceDispatcher;
    DiscoverDialog * const discoverDialog;
    RegOpModel * const regOpModel;
    double testValue = 0;
    quint64 curDevSerial;
};

#endif // DREMAINWINDOW_H
