#include "FileReader.h"

#include <QDebug>
#include <QDir>
#include <QSemaphore>
#include <QTimer>

namespace {
const qint64 PACK_SIZE_LIMIT = 20e6;
}

FileReader::FileReader(QSemaphore *sem):
    queueFreeSizeSem(sem),
    readFileTimer(new QTimer(this))
{
    connect(readFileTimer, SIGNAL(timeout()), SLOT(readFile()));
    readFileTimer->setSingleShot(false);
    readFileTimer->setInterval(1000);
    readFileTimer->start();
}


void FileReader::readFile()
{
    if(!dataFile.exists() || !dataFile.isReadable())
        return;
    bool idle=true;
    while(!dataFile.atEnd()){
        int sizeToRead = qMin(dataFile.size()-dataFile.pos(), PACK_SIZE_LIMIT);
        if(sizeToRead==0){
            break;
        }

        if(!queueFreeSizeSem->tryAcquire()){
            break;
        }

        emit dataAccuired(dataFile.read(sizeToRead));
        idle=false;
    }
    if(!idle && dataFile.atEnd())
        qWarning().noquote()
            << QString("[FileReader] Reach end of file:")+dataFile.fileName();
}


void FileReader::setFileName(QString fileName)
{
    qInfo().noquote()
        << QString("[FileReader] Start reading file:")+fileName;
    if(dataFile.isOpen())
        dataFile.close();
    dataFile.setFileName(fileName);
    dataFile.open(QIODevice::ReadOnly);
}
