#ifndef HRBPLOT_H
#define HRBPLOT_H

#include <QWidget>

namespace Ui {
class HrbPlot;
}

class QLabel;
class QwtPlot;
class QHostAddress;
class QSemaphore;

class HrbPlot : public QWidget
{
    Q_OBJECT

public:
    explicit HrbPlot(quint32 serialId, QWidget *parent = 0);
    ~HrbPlot();
    enum TYPE_DISPLAY {TYPE_CH=1, TYPE_SAMPLE=2, TYPE_INFO=3};

    quint32 getSerial() const { return serial; }
    QLabel *getStatusLabel() { return statusLabel; }
    QwtPlot *getChPlot() { return chPlot; }
    QwtPlot *getSamPlot() { return samPlot; }
    void setAddress(QHostAddress &, int port);
    QString getHostAddr();
    int getHostPort();
    void setAutoClear(bool checked);
    void disableAnalyzer();
    void updateCounters(QVector<QVector<quint64> > data, quint32 trigPassed, double histLimit);

private:
    void setupPlot(QwtPlot *plot);
    void replot(QwtPlot *plot, QVector<double> data, int xStep);

signals:
    void addLog(QString);
    void dataAccuired(int index, QByteArray);
    void connectedStateChanged(int index, bool state);
    void updateEnabled();
    void clear();
    void sumUpdated(quint64, quint32);

public slots:
    void initialize();
    void updateHistogram(int index, quint32 devId, QVector< QVector<quint64> > cur, quint32 trig);
    void updateStatus(bool status);
    void gropingUpdated(bool isByChannels);
    void gropingUpdated(int type);

private:
    Ui::HrbPlot *ui;

    const quint32 serial;
    QSemaphore *analyzerQueueSize;
    QLabel *statusLabel;
    QwtPlot *chPlot;
    QwtPlot *samPlot;
    quint32 curHrbIds;
    bool online;
    bool isGroppingByChannels;
};

#endif // HRBPLOT_H
