/*
**    Copyright 2013-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef HRBDEVICE_H
#define HRBDEVICE_H

#include <QVector>
#include <QMetaType>

#include "HrbStatus.h"
#include "ModularDeviceConfig.h"
#include "ModularMregDevice.h"

class LegacyDeviceModule;

class HrbDevice : public ModularMregDevice
{
    Q_OBJECT
public:
    explicit HrbDevice(const DeviceIndex &i, QObject *parent = nullptr);
    virtual void clearStatistics();
    QVector<quint32> readHitCounts();
    HrbStatus getStatus() const { return status; }
    ModularDeviceConfig getSetup() const { return config; }
    static int thresholdConverter(double val);

public slots:
    void writeThresholds(const QVector<int> &v);
    void softReset();
    void evNumRequest(int checkIndex) override;

protected slots:
    void on_deviceConnected() override;
    void on_deviceDisconnected() override;

signals:
    void hrbMStreamVer(const DeviceIndex &index, bool isMStream22) const;
    void statusUpdated(const HrbStatus &) const;
    void connectedStateChanged(const DeviceIndex &index, bool connected) const;

protected:
    bool readStatus() override;
    bool writeConfig() override;

    bool readTTCLinkStatus();
    bool readTTCLinkErrors();
    bool readHrbStatus();
    bool readStatistics();
    QString logPrefix() const { return getIdent(); }

    void updateCountersFile(QVector<quint32> counters);
    QVector<quint32> readCnt32(quint32 base, int count);
    void readHit2DHistogram();

    HrbStatus status;
    QSharedPointer<LegacyDeviceModule> legacyDevice;
};

#endif // HRBDEVICE_H
