#ifndef MONGOVIEWERMAINWINDOW_H
#define MONGOVIEWERMAINWINDOW_H

#include "configuration-manager/ConfigurationManager.h"
#include "MongoViewerConfig.h"
#include "app/BaseMainWindow.h"
#include "base/ClientIndex.h"
#include "base/RcRunId.h"

#include <QTreeWidgetItem>

namespace Ui {
class MongoViewerMainWindow;
}

class Viewer;
class MongoViewerConfigDB;
class MongoViewerConfigDispatcher;
class JsonTreeModel;
class QTableWidget;
class QSplitter;

struct RunMap {
    ClientIndex parent;
    QMap<ClientIndex, RunMap> children;
};

struct RunHistConfigRecord
{
    QJsonObject data;
    bool parse() {
        {};
        parsed = true;
        return parsed;
    }
    QJsonObject getHeader() const {return headerData;}
    QJsonObject getAppConfig(ClientIndex clientIndex) const {return appConfigs.value(clientIndex);}
    RunMap getRunMap() const { return runMap; }

    bool parsed = false;
    QMap<ClientIndex, QJsonObject> appConfigs;
    QJsonObject headerData;
    RcRunId rcRunId;
    RunMap runMap;
};

class MongoViewerMainWindow : public BaseMainWindow
{
    Q_OBJECT
public:
    explicit MongoViewerMainWindow(QWidget *parent = nullptr);
    ~MongoViewerMainWindow();
signals:
    void configChanged(const MongoViewerConfig &config);

public slots:
    void setConfig(const MongoViewerConfig &newConfig);

private slots:
    void on_host_navigate_itemDoubleClicked(QTreeWidgetItem *item, int column);
    void on_tableWidgetDB_itemSelectionChanged();
    void history_navigate_item_changed(QItemSelection item, QItemSelection prev);
    void on_pushButtonRefresh_clicked();
    void on_groupBoxShowPeriod_toggled(bool checked);
    void on_pushButtonLeftPage_clicked();
    void on_pushButtonRightPage_clicked();
    void on_lineEditPageNumber_textChanged(const QString &arg1);
    void on_lineEditPageNumber_editingFinished();
    void on_lineEditRowsOnPage_textChanged(const QString &arg1);
    void on_lineEditRowsOnPage_editingFinished();
    void on_comboBoxProgramType_currentIndexChanged(const QString &arg1);
    void on_comboBoxProgramIndex_currentIndexChanged(const QString &arg1);
    void on_groupBoxRunNumber_toggled(bool checked);
    void on_spinBoxRunNumberFrom_valueChanged(int arg1);
    void on_spinBoxRunNumberFrom_editingFinished();
    void on_spinBoxRunNumberTo_valueChanged(int arg1);
    void on_spinBoxRunNumberTo_editingFinished();
    void on_radioButtonExpand_toggled(bool checked);
    void on_pushButtonDelete_clicked();
    void on_dateTimeEditFrom_dateTimeChanged(const QDateTime &dateTime);
    void on_dateTimeEditTo_dateTimeChanged(const QDateTime &dateTime);
    void on_checkBoxCloseHN_toggled(bool checked);
    void on_actionShow_all_collections_toggled(bool arg1);

    // TODEL
    void on_pushButton_3_clicked();


    void on_pushButton_clicked();

private:
    void init_gui();
    void update_gui();
    void hide_history_navigate();
    void set_style_sheet();
    void refresh_all();
    void refresh_page();
    void populate_filters();
    void populate_filters_history();
    void populate_filters_saved();

    void split_history();
    void populate_host_navigation();
    void populate_table_db();
    void populate_tree_viewer(QJsonObject obj);
    void populate_history_navigate();
    void clear_qtablewidget();

    QJsonObject history_navigation_recurse(QJsonObject value);
    void history_map_recurse(QJsonObject obj);

    void set_visible_filters();
    void resize_line_edit_to_content(QLineEdit * line_edit);
    void check_left_right_buttons();
    void change_refresh_button_color(QColor color);
    void block_widgets_signal(bool block);
    QJsonObject readble_dates(QJsonObject obj);
    QString get_oid(QJsonObject &obj);

    Ui::MongoViewerMainWindow *ui;
    MongoViewerConfig config;
    MongoViewerConfigDispatcher * const config_dispatcher;
    MongoViewerConfigDB * const config_db;
    Viewer * const viewer;
    QList<QWidget*> widgets_to_block;
    QJsonObject history_navigation;
    QMap<QString, QJsonObject> history_map;
    QString history_map_key;
    QSplitter *split_gorisontal_history;
    JsonTreeModel * const tree_model;
    JsonTreeModel * const history_navigate_model;
    JsonTreeModel * const main_model;
};

#endif // MONGOVIEWERMAINWINDOW_H
