#ifndef EVENTBUILDERFSM_H
#define EVENTBUILDERFSM_H

#include "fsm/BaseFsm.h"
#include "base/ClientIndexEnabled.h"

struct EventBuilderFsmInput : public BaseFsmInput
{
    FsmState combinerState = FsmState::Unknown;
    FsmState clientManagerFsmState = FsmState::Unknown;
    bool lookForStopEvNum = false;
};

class  EventBuilderFsm : public BaseFsm
{
    Q_OBJECT
public:
    EventBuilderFsm(QObject *parent = nullptr);
    virtual bool isTransitionAllowed(const FsmState &toState) const override;

public slots:
    void setCombinerState(FsmState cur_state);
    void setClientManagerFsmState(FsmState cur_state);
    void setLookForStopEvNum(bool lookForStopEvNum);

protected slots:
    virtual bool evaluate() override;

protected:
    virtual void printInputString(QDebug dbg) const override;
    virtual qint64 stateTimeoutMs(const FsmState &state) const override;
    inline EventBuilderFsmInput *getInput() {
        return static_cast<EventBuilderFsmInput*>(BaseFsm::getInput());
    }
    inline const EventBuilderFsmInput *getInput() const {
        return static_cast<const EventBuilderFsmInput*>(BaseFsm::getInput());
    }

signals:
    void doStateAction(const FsmState &state) const;

private:
    bool allInState(FsmState state) const;
    bool hasError() const;
};

QDebug operator<<(QDebug dbg, const EventBuilderFsmInput &in);

#endif // EVENTBUILDERFSM_H
