#ifndef RINGBUFFER_H
#define RINGBUFFER_H

#include <QVector>
#include <QSemaphore>

template<class T> class RingBuffer
{
public:
    RingBuffer(int _size);
    int getAllSize() const { return size; }
    QString getStrStatus() const { return QString("%1/%2/%3")
                .arg(freeSpace.available())
                .arg(dataSpace.available())
                .arg(trashSpace.available()); }
    void clear();
    bool resize(qint64 newSize);
    // =========== Writer part ===========
    int getFreeSize() const;
    double getFreePart();
    int getContinuosFreeSize() const;
    void writeData(int continuousSize=1);
    // pass
    bool waitFreeSpace(int s, int timeout);
    T *getWritePointer();
    int getWriterPos() const;

    // =========== Reader part ===========
    int getDataSize() const; // reader
    double getDataPart();
    int getContinuosDataSize(int shift) const; // reader
    void trashData(int continuousSize=1); // writer
    void trashData(int continuousSize, int checkReaderPos); // writer
    // pass
    bool waitDataSpace(int s, int timeout);
    T *getReadPointer(int shift=0); // reader
    int getReaderPos(int shift=0) const;
    void readData(char *dst, int shift, int s); //reader // can read splited data; s>0

    // =========== Cleaner part ===========
    int getTrashSize() const;
    double getTrashPart();
    int getContinuosTrashSize() const;
//    int getContinuosTrashSize();
    void freeData(int continuousSize, int checkTrashPos); // reader // free only continious data
    void freeData(int continuousSize=1); // reader // free only continious data
    // pass
    bool waitTrashSpace(int s, int timeout);
    T *getClearPointer(int shift=0); // reader

private:
    // =========== Uniform part ===========
    int getContinuosTypeSize(int type, int shift=0) const; // free data
    bool waitTypeSize(int type, int s, int timeout); // free data trash
    void addTypeData(int type, int continuousSize, int pos); // free      trash
    void addTypeData(int type, int continuousSize); // free data trash
    T *getTypePointer(int type, int shift=0); // free data trash
    int getTypePos(int type, int shift=0) const; // free data trash

    qint64 size; // max size that we can store, ringData will have one extra item

    std::vector<T> ringData; // ringData.size()=size+1
    qint64 writePos; // data position, where we start writing
    qint64 readPos; // data position that will be transfered (if it is not writePos)
    qint64 trashPos; // data position that will be cleaned up (if it is not readPos)

    QSemaphore freeSpace;
    QSemaphore dataSpace;
    QSemaphore trashSpace;
};

#include "RingBuffer.tpp"

#endif // RINGBUFFER_H
