#include "MpdTluMainWindow.h"
#include "ui_MpdTluMainWindow.h"

#include "MpdTluSignalRouter.h"
#include "TluTtlIoDialog.h"
#include "Tlu16Sfp/Tlu16SfpModel.h"
#include "Tlu40Lvds/Tlu40LvdsModel.h"

#include "base/Globals.h"

#include <QSettings>

namespace {
const int STATUS_UPDATE_DELAY = 1000;
}

MpdTluMainWindow::MpdTluMainWindow(QWidget *parent)
    : ReadoutAppMainWindow(parent)
    , ui(new Ui::MpdTluMainWindow)
    , tluTtlIoDialog(new TluTtlIoDialog(this))
{
    ui->setupUi(this);
    ui->pushButtonStart->setVisible(false);
    ui->pushButtonStop->setVisible(false);
    setupMenu(ui->menuFile, ui->menuHelp, ui->menuOptions, ui->menuTools);

    init_connections();

    updateControlState();

    auto signalRouter = &MpdTluSignalRouter::instance();
    signalRouter->setMainWindow(this);
}

MpdTluMainWindow::~MpdTluMainWindow()
{
    delete ui;
}

DeviceStatusTableViewFancy* MpdTluMainWindow::getDeviceStatusTableView()
{
    return ui->deviceStatusTableView;
}

Tlu16SfpModel *MpdTluMainWindow::getTlu16SfpModel() const
{
    return ui->widget16Sfp->getModel();
}

Tlu40LvdsModel *MpdTluMainWindow::getTlu40LvdsModel() const
{
    return ui->widget40Lvds->getModel();
}

TluTtlIoWidget *MpdTluMainWindow::getTluTtlIoWidget40Lvds() const
{
    return tluTtlIoDialog->getWidget40Lvds();
}

TluTtlIoWidget *MpdTluMainWindow::getTluTtlIoWidget16Sfp() const
{
    return tluTtlIoDialog->getWidget16Sfp();
}

void MpdTluMainWindow::init_connections()
{
    connect(ui->actionPrint_FSM_inputs, &QAction::triggered,
            this, &MpdTluMainWindow::printFsmInputs);
    connect(ui->actionConfigure_TLU_TTL_IO, &QAction::triggered,
            tluTtlIoDialog, &TluTtlIoDialog::show);
    connect(ui->pushButtonResetCntLvds, &QPushButton::clicked,
            this, &MpdTluMainWindow::resetCntLvds);
    connect(ui->pushButtonResetCntSfp, &QPushButton::clicked,
            this, &MpdTluMainWindow::resetCntSfp);
}

void MpdTluMainWindow::saveAppGuiSettings(QSettings &s) const
{
    BaseMainWindow::saveAppGuiSettings(s);
    s.beginGroup("MW");
    s.setValue("showStartStop", ui->actionShow_Start_Stop_button->isChecked());
    s.endGroup();
}

void MpdTluMainWindow::loadAppGuiSettings(const QSettings &s)
{
    BaseMainWindow::loadAppGuiSettings(s);
    bool showVar = s.value("MW/showStartStop").toBool();
    ui->actionShow_Start_Stop_button->setChecked(showVar);
}

void MpdTluMainWindow::updateControlState()
{
    ui->pushButtonStart->setEnabled(curFsmState == FsmState::Ready && !underRemoteCtrl);
    ui->pushButtonStop->setEnabled(curFsmState == FsmState::Run && !underRemoteCtrl);

    bool en = Globals::getDebugMode() || (curFsmState == FsmState::Error || curFsmState == FsmState::Ready);
    if(auto m=getTlu16SfpModel())
        m->setEnabled(en);
    if(auto m=getTlu40LvdsModel())
        m->setEnabled(en);
}

bool MpdTluMainWindow::isControlBlocked() const
{
    return underRemoteCtrl;
}

void MpdTluMainWindow::setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig)
{
    Q_UNUSED(config_key)
    if(config == newConfig)
        return;

    ReadoutAppMainWindow::setConfig(config_key, newConfig);

    updateControlState();
}

// ==== Slots from UI

void MpdTluMainWindow::on_pushButtonStart_clicked()
{
    if(isControlBlocked()) return;
    if(!config.hasEnabledDevices()) {
        qWarning() << "There is no enabled device";
        return;
    }

    emit performLocalStart();
}

void MpdTluMainWindow::on_pushButtonStop_clicked()
{
    if(isControlBlocked()) return;
    emit performLocalStop();
}

void MpdTluMainWindow::on_actionShow_Start_Stop_button_toggled(bool showPb)
{
    ui->pushButtonStart->setVisible(showPb);
    ui->pushButtonStop->setVisible(showPb);
}
