#ifndef TLU16SFPMODEL_H
#define TLU16SFPMODEL_H

#include <QAbstractItemModel>
#include <QModelIndex>
#include <QVariant>
#include <optional>

#include "base/DeviceIndex.h"
#include "lib-common/BaseDeviceAppConfig.h"

#include "Tlu16SfpModelItem.h"
#include "Tlu16SfpTrigger/Tlu16SfpTriggerConfig.h"
#include "Tlu16SfpTrigger/Tlu16SfpTriggerBuildParams.h"
#include "TluDecDes/TluDecDesBuildParams.h"
#include "TluDecDes/TluDecDesStatus.h"

class QTimer;
class ConfigKey;

class Tlu16SfpModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    Tlu16SfpModel(QObject *parent = nullptr);

    QVariant data(const QModelIndex &index, int role) const override;
    virtual bool setData(const QModelIndex &index,
                         const QVariant &value,
                         int role = Qt::EditRole) override;
    Qt::ItemFlags flags(const QModelIndex &index) const override;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role) const override;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const override;
    QModelIndex parent(const QModelIndex &index) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    void setEnabled(bool en);
    int getDevRow(const DeviceIndex &di) const;

signals:
    void configChanged(const BaseDeviceAppConfig &config) const;

public slots:
    void setConfig(const ConfigKey &config_key,
                   const BaseDeviceAppConfig &newConfig);
    void setTlu16SfpTrigBp(const DeviceIndex &index,
                           const Tlu16SfpTriggerBuildParams &bp);
    void setTluDesDecBp(const DeviceIndex &index,
                        const TluDecDesBuildParams &bp);
    void setTluDecDesStatus(const DeviceIndex &index,
                            const TluDecDesStatus &status);
    void resetCounters();
    void onDeviceConnected(const DeviceIndex &devIndex);

private slots:
    void postCheckStateChange();

private:
    void enqueueConfigSend();
    void doReset();
    void sendDataChanged(QVector<int> roles = QVector<int>(), const QModelIndex parent = {});

    QVariant displayData(const QModelIndex &index) const;
    QVariant editData(const QModelIndex &index) const;
    QVariant toolTipData(const QModelIndex &index) const;
    QVariant checkStateData(const QModelIndex &index) const;
    QVariant foregroundData(const QModelIndex &index) const;
    QVariant limitsData(const QModelIndex &index) const;

    inline Tlu16SfpModelItem *getItemByIndex(const QModelIndex &index) const;
    QVariant getRecursiveState(const QModelIndex &mIndex) const;
    const TluDecDesChStatus *getChStatusPtr(Tlu16SfpModelItem *mItem, bool offset = false) const;
    QPair<std::optional<quint32>, std::optional<quint32>> getCounterPair(Tlu16SfpModelItem *item) const;

    BaseDeviceAppConfig config;
    QMap<DeviceIndex, Tlu16SfpTriggerBuildParams> buildParamsTrig;
    QMap<DeviceIndex, TluDecDesBuildParams> buildParamsDecDes;
    QMap<DeviceIndex, TluDecDesStatus> devStatus;
    QMap<DeviceIndex, TluDecDesStatus> devStatusOffset;
    Tlu16SfpModelItem rootItem;
    QTimer *const delayedSendConfigTimer;
    bool enable = true;
};

#endif // TLU16SFPMODEL_H
