/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef RCCONFIGDISPATCHER_H
#define RCCONFIGDISPATCHER_H

#include "lib-common/BaseConfigDispatcher.h"

struct RcConfig;
class RcConfigDispatcherPrivate;
struct SlowControlConfig;
struct SpillConfig;
struct ClientIndexEnabledMap;
struct RcClientConfigMap;

class RcConfigDispatcher : public BaseConfigDispatcher
{
    Q_OBJECT
public:
    explicit RcConfigDispatcher(QObject *parent = nullptr);
    ~RcConfigDispatcher() override;
    virtual void sendCache() override;

    template<class T>
    void subscribe(T *obj) {
            connect(this, &RcConfigDispatcher::configChanged,
                    obj, &T::setConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::configChanged,
                    this, &RcConfigDispatcher::setConfig,
                    Qt::UniqueConnection);
            logConnection(obj, "Config");
            sendCache();
    }

signals: void configChanged(const RcConfig &) const;
public slots: void setConfig(const RcConfig &newConfig);

public:
    template<class T>
    void subscribeRcClientConfigMap(T *obj) {
            connect(this, &RcConfigDispatcher::rcClientsConfigChanged,
                    obj, &T::setRcClientsConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::rcClientsConfigChanged,
                    this, &RcConfigDispatcher::setRcClientsConfig,
                    Qt::UniqueConnection);
            logConnection(obj, "RcClientConfigMap");
            sendCache();
    }
signals: void rcClientsConfigChanged(const RcClientConfigMap &) const;
public slots: void setRcClientsConfig(const RcClientConfigMap &newConfig);

public:
    template<class T>
    void subscribeSlowControl(T *obj) {
            connect(this, &RcConfigDispatcher::slowControlConfigChanged,
                    obj, &T::setSlowControlConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::slowControlConfigChanged,
                    this, &RcConfigDispatcher::setSlowControlConfig,
                    Qt::UniqueConnection);
            logConnection(obj, "SlowControlConfig");
            sendCache();
    }
signals: void slowControlConfigChanged(const SlowControlConfig &) const;
public slots: void setSlowControlConfig(const SlowControlConfig &newConfig);

public:
    template<class T>
    void subscribeSpillConfig(T *obj) {
            connect(this, &RcConfigDispatcher::spillCfgChanged,
                    obj, &T::setSpillCfg,
                    Qt::UniqueConnection);

            connect(obj, &T::spillCfgChanged,
                    this, &RcConfigDispatcher::setSpillCfg,
                    Qt::UniqueConnection);
            logConnection(obj, "SpillConfig");
            sendCache();
    }
signals: void spillCfgChanged(const SpillConfig &) const;
public slots: void setSpillCfg(const SpillConfig &cfg);

public slots:
    virtual void setDlnpCalibrationTriggerConfig(const DlnpCalibrationTriggerConfig &cfg) override;

protected:
    DeviceIndexEnabledMap getDeviceIndexEnabledMap() const override;
    BaseDeviceAppConfig &getAppConfigCache() override;

private:
    Q_DECLARE_PRIVATE(RcConfigDispatcher)
    RcConfigDispatcherPrivate * const d_ptr;
};

#endif // RCCONFIGDISPATCHER_H
