#include "VmeTrigFsm.h"

#include <QDebug>
#include <QLoggingCategory>

namespace {
    Q_DECLARE_LOGGING_CATEGORY(fsm)
    Q_LOGGING_CATEGORY(fsm, "afi.fsm.vme_device")
}

VmeTrigFsm::VmeTrigFsm(QObject *p)
    : DeviceMgrFsm(p)
{
    setObjectName(__func__);
    recreateInput(new VmeTrigFsmInput());
}

void VmeTrigFsm::setWrState(const DeviceIndex &index, bool ok)
{
    auto it = getInput()->wrOk.find(index);
    if(it == getInput()->wrOk.end())
        getInput()->wrOk[index] = ok;
    else if(*it != ok)
        *it = ok;
    else
        return;
    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << index << ok;
    evaluate();
}

void VmeTrigFsm::setWrCheckSet(DeviceIndexSet wrCheckSet)
{
    if(getInput()->wrCheckSet == wrCheckSet)
        return;
    getInput()->wrCheckSet = wrCheckSet;
    evaluate();
}

bool VmeTrigFsm::evaluate()
{
    bool wrOk = allWrOk();
    if(!wrOk) {
        setState(FsmState::Error);
        return true;
    }

    if(DeviceMgrFsm::evaluate())
        return true;

    switch (getState()) {
    case FsmState::Starting:
        if(setRequestState(FsmState::Run))
            return true;
        break;
    case FsmState::Stopping:
        if(setRequestState(FsmState::Ready))
            return true;
        break;
    default:;
    }

    return false;
}

void VmeTrigFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

bool VmeTrigFsm::allWrOk()
{
    for(auto devIndex : getInput()->wrCheckSet) {
        if(!getInput()->wrOk.value(devIndex, false))
            return false;
    }
    return true;
}

QDebug operator<<(QDebug dbg, const VmeTrigFsmInput &in)
{
    dbg << DeviceMgrFsmInput(in);

    dbg << "wrCheckSet:" << in.wrCheckSet << "; ";
    dbg << "wrOk:" << in.wrOk << "; ";

    return dbg;
}
