#ifndef CALIBRATIONMODEL_H
#define CALIBRATIONMODEL_H

#include <QAbstractItemModel>

#include <QMap>

#include "cal/CalResult.h"
#include "cal/CalConfig.h"
#include "base/DeviceIndexEnabled.h"

class ConfigKey;
struct DeviceChModelItem;

class CalibrationModel : public QAbstractItemModel
{
    Q_OBJECT
    enum class TreeColumns{
        NAME,
        BASELINE, BASELINE_RMS,
        BASELINE4, BASELINE4_RMS,
        DAC, TOTAL
    };
public:
    CalibrationModel(QObject *parent = nullptr);
    void clear();
    void reset();
    void removeDev(const DeviceIndex &index);

    QVariant data(const QModelIndex &index, int role) const override;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role) const override;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const override;
    QModelIndex parent(const QModelIndex &index) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override
    {
        Q_UNUSED(parent)
        return static_cast<int>(TreeColumns::TOTAL);
    }

signals:
    void deviceListChanged(const DeviceIndexEnabledMap &);
    void calConfigChanged(const CalConfig &) const;

public slots:
    void setCalConfig(const ConfigKey &config_key, const CalConfig &newCalConfig);
    void setDeviceList(const DeviceIndexEnabledMap &map);

private:
    QVariant displayData(const QModelIndex &index) const;
    QVariant getForegroundData(const QModelIndex &index) const;
    QVariant fontData(const QModelIndex &index) const;
    QVariant getHeaderForeground(TreeColumns col) const;
    bool isDeviceRmsOk(const DeviceIndex &index) const;

    inline TreeColumns getColumnByIndex(const QModelIndex &index) const;
    inline DeviceChModelItem *getItemByIndex(const QModelIndex &index) const;
    int getDevRow(const DeviceIndex &di);
    void resetItems();

    DeviceChModelItem * const rootItem;
    CalConfig cal_config;
    DeviceIndexEnabledMap map;
};

#endif // CALIBRATIONMODEL_H
