#ifndef TRCCORE_H
#define TRCCORE_H

#include <QObject>
#include <QSharedPointer>

#include <optional>

#include "TrcDeviceGlobal.h"
#include "base/DeviceIndex.h"
#include "base/RcRunId.h"
#include "fsm/BaseFsm.h"
#include "fsm/FsmState.h"
#include "lib-common/BaseDeviceAppConfig.h"
#include "lib-common/ReadoutAppCore.h"
#include "mldiscover/LldpDiscovery.h"
#include "remote-control-server/RemoteControlServerStatus.h"

class ConfigKey;
class MStreamDeviceMgrFsm;
class ReadoutAppConfigDb;
class TrcConfigDispatcher;
struct DeviceIndexEnabledMap;

class TrcCore : public ReadoutAppCore
{
    Q_OBJECT
public:
    explicit TrcCore(QObject *parent = nullptr);

signals:
    void configChanged(const BaseDeviceAppConfig &config) const;
    void blockDeviceRun(bool isBlock);
    void preRunReset(const QVariant &startEvNum);
    void prepareForStop(quint32 stopEvNum);
    void updateControlState();
    void remoteCompletedStart(bool success, const RcRunId &runId);
    void remoteCompletedStop(bool success, const RcRunId &runId);
    void mstreamResetAll(int mstreamSeqNum) const;

public slots:
    void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig);
    void performLocalStart();
    void performLocalStop();
    void remoteStartRequest(const RcRunId &newRunId, const QVariant &startEvNum);
    void remoteStopRequest(const RcRunId &newRunId, const QVariant &stopEvNum);
    void onRemoteStatusUpdated(const RemoteControlServerStatus &status);
    void onFsmStateChange(const FsmState &state, const FsmState &oldState) override;
    void deviceDisconnected(const DeviceIndex &index);
    void deviceStatusUpdated(const TrcStatus &status);

private:
    virtual const BaseFsm *getFsm() const override;
    void initTrcProgDescr();
    void updateTrcProgDescrOptions();
    void addDevice(const DeviceIndex &index) override;
    void removeDevice(const DeviceIndex &index) override;
    void sendRunDocument() const;

    QVariant runStartEvNum;
    MStreamDeviceMgrFsm  * const deviceMgrFsm;
    ReadoutAppConfigDb * const configDb;
    BaseDeviceAppConfig config;
    LldpInfoCache lldpCache;
    QMap<DeviceIndex, QSharedPointer<TrcDeviceGlobal> > devices;
    QMap<DeviceIndex, TrcStatus> dev_status;
    RemoteControlServerStatus lastRcStatus;
    int mstreamSeqNum = 0;
};

#endif // TRCCORE_H
