#ifndef TRCMAINWINDOW_H
#define TRCMAINWINDOW_H

#include <QElapsedTimer>
#include <QLabel>

#include "TrcStatus.h"

#include "lib-common/ReadoutAppMainWindow.h"
#include "mldiscover/LldpDiscovery.h"

namespace Ui {
class TrcMainWindow;
}

class QTableWidget;
class QTableWidgetItem;
//class QTableView;

class CalibrationModel;
class ConfigKey;
class RcRunId;
class RemoteControlServerStatus;
class TrcCountersDialog;
class TrcDeviceChModel;
struct CalibrationTriggerConfig;
struct DeviceIndexEnabledMap;
struct MStreamStat;

class TrcMainWindow : public ReadoutAppMainWindow
{
    Q_OBJECT

public:
    explicit TrcMainWindow(QWidget *parent = nullptr);
    ~TrcMainWindow() override;
    DeviceStatusTableViewFancy* getDeviceStatusTableView() override;

signals:
    void softReset();
    void softClear();
    void prepareForStop(quint32 stopEvNum);
    void mstream_reset_all(bool doStatClear);
    void performLocalStart() const;
    void performLocalStop() const;
    void printFsmInputs() const;

public slots:
    void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig) override;
    void lldpCacheUpdated(const LldpInfoCache &cache);
    void deviceStatusUpdated(const TrcStatus &status);
    void deviceConnected(const DeviceIndex &index);
    void deviceDisconnected(const DeviceIndex &index);
    void update_mstream_data_rate(const MStreamStat &ms_stat);

protected:
    void saveAppGuiSettings(QSettings &) const override;
    void loadAppGuiSettings(const QSettings &) override;
    void updateControlState() override;

private slots:
    void on_pushButtonStart_clicked();
    void on_pushButtonStop_clicked();

    void on_actionShow_experts_settings_toggled(bool showExperts);
    void on_actionCounters_dialog_triggered();
    void on_actionReset_MStream_triggered();
    void on_actionSend_soft_reset_triggered();
    void on_actionSend_soft_clear_triggered();

    void on_spinBoxMatchWin_valueChanged(int mw);
    void on_spinBoxTrigDelay_valueChanged(int delay);
    void on_radioButtonOscDelayed_toggled(bool checked);
    void on_groupBoxChWaveform_toggled(bool checked);
    void on_spinBoxLatency_valueChanged(int lat);
    void on_spinBoxReadPoints_valueChanged(int points);

    void on_groupBoxCalibTrigger_cfgUpdated(const CalibrationTriggerConfig &c);
    void on_tableWidgetDevice_currentItemChanged(QTableWidgetItem *current, QTableWidgetItem *previous);

private:
    DeviceIndex getIndexByRow(int row);
    int getDevRow(const DeviceIndex &index);
    void init_connections();
    void init_table();
    void copyAndSetupDevice();
    void setupDevice();
    void updateGuiVal();
    void controlStateEnable(bool enable);
    void disconnectFromDev(const DeviceIndex &index);
    void addDevice(const DeviceIndex &index);
    void temperatureSet(DeviceIndex index, QMap<QString, double> temps);
    void setTableDevRowCnt(int cnt);
    void deviceStatusUpdatedRequest();
    void deviceStatusUpdatedResponce();
    bool isControlBlocked() const;

public:
    static void updateTableSize(QTableWidget * const table,
                                bool widthOnly=false);

private:
    DeviceIndex getDevIndexByRow(int row) const;
    void updateDevWrStatus(int row, const std::optional<WrStatus> &wrStatus);
    QColor bgColorDefault() const;
    QColor bgColorWarning() const;
    QColor bgColorError() const;

    Ui::TrcMainWindow *ui;
    QMap<DeviceIndex, TrcStatus> dev_status;
    QMap<DeviceIndex, MStreamStat> prev_ms_stat;
    int blockControl = 0;

    bool isDevBlocked = false;
    TrcDeviceChModel * const trcModel;
    QTimer *status_update_timer;
    bool status_request_flag = false;
    LldpInfoCache lldpCache;
//    RcCountersModel * const rcCountersModel;
//    QTableView * const rcView;
    TrcCountersDialog * const countersDialog;
};

#endif // TRCMAINWINDOW_H
