/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "TtvxsAppConfigDispatcher.h"

#include "lib-common/RunControlAppConfig.h"

TtvxsAppConfigDispatcher::TtvxsAppConfigDispatcher(QObject *parent) :
    BaseConfigDispatcher(parent),
    cache(new RunControlAppConfig())
{
    qRegisterMetaType<RunControlAppConfig>();
}

DeviceIndexEnabledMap TtvxsAppConfigDispatcher::getDeviceIndexEnabledMap() const
{
    return cache->getDeviceIndexEnabledMap();
}

BaseDeviceAppConfig &TtvxsAppConfigDispatcher::getAppConfigCache()
{
    return *cache;
}

void TtvxsAppConfigDispatcher::sendConfigKeyCache()
{
    BaseConfigDispatcher::sendConfigKeyCache();
    emit configChanged(config_key, *cache);
    emit rcClientsConfigChanged(cache->clients);
}

void TtvxsAppConfigDispatcher::setConfig(const RunControlAppConfig &config)
{
    if (config == *cache)
        return;
    *cache = config;
    incrementConfigKey(true);
}

void TtvxsAppConfigDispatcher::setConfigFeLink(const BaseDeviceAppConfig &config)
{
    if (config.knownConfigs == cache->knownConfigs)
        return;
    cache->knownConfigs = config.knownConfigs;
    incrementConfigKey(true);
}

void TtvxsAppConfigDispatcher::setRcClientsConfig(const RcClientConfigMap &newConfig)
{
    if (newConfig == cache->clients)
        return;
    cache->clients = newConfig;
    incrementConfigKey();
    emit rcClientsConfigChanged(newConfig);
}
