#include "AfiViewerConfigDb.h"

#include <QLoggingCategory>

#include "AfiViewerConfigConverter.h"

AfiViewerConfigDb::AfiViewerConfigDb(const AfiViewerStartupOptions &_startupOptions, QObject *parent):
    AbstractConfigDb(parent),
    startupOptions(_startupOptions)
{
    AbstractConfigConverter<AfiViewerConfig>::registerTypes();
}

void AfiViewerConfigDb::setConfig(const AfiViewerConfig &newConfig)
{
    if(!initialized)
        return;
    if (newConfig == config)
        return;
    config = newConfig;

    auto json = QVariant::fromValue(config).value<QJsonObject>();
    write_config(json);
}

void AfiViewerConfigDb::update_config_from_db()
{
    auto json = readConfig(cs);
    config = QVariant::fromValue(json).value<AfiViewerConfig>();
    if(!startupOptions.fileName.isNull() && !startupOptionsUpplied) {
        config.source = AfiViewerConfig::SOURCE_DATA::SOURCE_FILE;
        config.file_reader_config.lastFile = startupOptions.fileName;
        startupOptionsUpplied = true;
    }
    config.hist_manager_config.init(getType(), startupOptions.inl);
    config.hist_manager_config.makeConfigValid();

    emit configChanged(config);
}
