#include "AfiViewerSignalRouter.h"

#include "ViewerMainWindow.h"
#include "AfiViewerConfigDb.h"
#include "AfiViewerConfigDispatcher.h"
#include "WaveformViewer.h"

#include "pnp-server/PnpDiscovery.h"
#include "mldiscover/LldpDiscovery.h"
#include "configuration-manager/ConfigurationManager.h"
#include "file-reader-dialog/FileReaderDialog.h"
#include "histogramm/HistWorker.h"
#include "histogramm/HistMngrWidget.h"
#include "zmq-lib/ZmqDialog.h"
#include "zmq-lib/ZmqPublisher.h"

namespace {
template <typename Func1, typename Func2>
inline void connect(const typename QtPrivate::FunctionPointer<Func1>::Object *sender, Func1 signal,
                    const typename QtPrivate::FunctionPointer<Func2>::Object *receiver, Func2 slot)
{
    QObject::connect(sender, signal, receiver, slot);
}
}

void AfiViewerSignalRouter::connectAll()
{
    Q_ASSERT(configDispatcher);

    connect_ConfigurationDispatcher();
    connect_ViewerMainWindow();
    if(mainWindow)
        PnpDiscovery::subscribeCacheUpdaed(mainWindow);
    connect_HistWorker();
    connect_HistWidget();
    connect_ZMQ();
}

void AfiViewerSignalRouter::connect_ViewerMainWindow()
{

}

void AfiViewerSignalRouter::connect_ConfigurationDispatcher()
{
    if(!configDispatcher)
        return;
    if(mainWindow) {
        configDispatcher->subscribe(mainWindow);
        configDispatcher->subscribeCS(mainWindow);
        if(auto v = mainWindow->getFileReaderDialog())
            configDispatcher->subscribeFileReaderCfg(v);
        if(auto v = mainWindow->getWaveformViewer())
            configDispatcher->subscribeWaveformViewerCfg(v);
    }
    if(configurationManager)
        configDispatcher->subscribeCS(configurationManager);
    if(configDb) {
        configDispatcher->subscribe(configDb);
        configDispatcher->subscribeCS(configDb);
    }
    if(histWorker)
        configDispatcher->subscribeHistCfg(histWorker);
    if(histMngrWidget)
        configDispatcher->subscribeHistCfg(histMngrWidget);
}

void AfiViewerSignalRouter::connect_HistWorker()
{
    if(!histWorker)
        return;

    if(mainWindow) {
        connect(mainWindow, &ViewerMainWindow::inlConfigUpdated,
                histWorker, &HistWorker::inlConfigUpdated);
        connect(mainWindow, &ViewerMainWindow::tdcTtvxMapUpdated,
                histWorker, &HistWorker::tdcTtvxMapUpdated);
        connect(mainWindow, &ViewerMainWindow::requestInlResults,
                histWorker, &HistWorker::requestInlResults);
        connect(histWorker, &HistWorker::inlResultsUpdated,
                mainWindow, &ViewerMainWindow::inl_corrections_updated);
    }

    if(histMngrWidget) {
        connect(histMngrWidget, &HistMngrWidget::clearHist,
                histWorker, &HistWorker::clearHist);
        connect(histWorker, &HistWorker::devListUpdated,
                histMngrWidget, &HistMngrWidget::devListUpdated);
        connect(histWorker, &HistWorker::hist_updated,
                histMngrWidget, &HistMngrWidget::hist_updated);
        connect(histWorker, &HistWorker::taiReperUpdated,
                histMngrWidget, &HistMngrWidget::taiReperUpdated);
    }
}

void AfiViewerSignalRouter::connect_HistWidget()
{
    if(!histWorker || !histMngrWidget)
        return;
}

void AfiViewerSignalRouter::connect_ZMQ()
{
    if(configDispatcher) {
        if(zmqDialog)
            configDispatcher->subscribeZmqCfg(zmqDialog);
        if(zmqPublisher)
            configDispatcher->subscribeZmqCfg(zmqPublisher);
    }
    if(zmqDialog && zmqPublisher)
        connect(zmqPublisher, &ZmqPublisher::statusUpdated,
                zmqDialog, &ZmqDialog::statusUpdated);
}
