#ifndef TCPCLIENT_H
#define TCPCLIENT_H

#include <QElapsedTimer>
#include <QHostAddress>
#include <QTimer>

#include "TcpClientStatus.h"

class QTcpSocket;
class QSemaphore;

class TcpClient : public QObject
{
    Q_OBJECT
public:
    TcpClient(QSemaphore *analyzerSem, QObject *parent=nullptr);
    ~TcpClient();

public slots:
    void setAddress(QHostAddress, quint16 port);

private slots:
    void requestNewData();
    void readData();
    void displayError(QAbstractSocket::SocketError socketError);
    void disconnected();
    void sendStatus();

signals:
    void dataAccuired(QByteArray);
    void addLog(QString);
    void statusUpdated(const TcpClientStatus &state) const;

private:
    void startReadOut();
    void stopReadOut();

    QSemaphore * const analyzerSem; // free space in analyzer queue
    bool enabled;
    QTcpSocket * const tcpSocket;
    QHostAddress hostAddr;
    quint16 hostPort;
    QTimer reconnectTimer;
    QTimer delayReadTimer;
    QTimer statusTimer;
    int connectingAttempts;
    QElapsedTimer statusPeriod;
    qint64 dataRecieved;
    qint64 lastDataRecieved;
    TcpClientStatus status;
};

#endif // TCPCLIENT_H
