/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "AbstractDispatcher.h"

#include <QDebug>
#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(lc_dispatch)
Q_LOGGING_CATEGORY(lc_dispatch, "afi.base.dispatch")
}

QString AbstractDispatcher::objectPrettyName(const QObject *obj)
{
    if (!obj)
        return "(null)";
    if (obj->objectName().isEmpty())
        return obj->metaObject()->className();
    else
        return obj->objectName();
}

void AbstractDispatcher::logConnection(const QObject *obj, const QString &name, const QString &symbol) const
{
    qDebug(lc_dispatch).noquote()
        << name
        << objectPrettyName(this)
        << symbol
        << objectPrettyName(obj);
}
