/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "DeviceIndexEnabled.h"

#include <QDebug>

QDebug operator<<(QDebug dbg, const DeviceIndexEnabled &r)
{
    dbg.noquote() << QString("{index:\"%1\", enabled:%2}")
           .arg(r.index.getIdent())
           .arg(r.enabled);
    return dbg;
}

DeviceIndexEnabledMap::operator DeviceIndexSet() const
{
    return makeDeviceIndexSet(this->keys());
}

DeviceIndexEnabledMap::operator DeviceIndexEnabledVector() const
{
    DeviceIndexEnabledVector result;
    for (auto it = begin(); it != end(); ++it)
        result.push_back(it.value());
    return result;
}

DeviceIndexEnabledVector::operator DeviceIndexSet() const
{
    DeviceIndexSet result;
    for (auto it = begin(); it != end(); ++it)
        result.insert(it->index);
    return result;
}

DeviceIndexEnabledVector::operator DeviceIndexEnabledMap() const
{
    DeviceIndexEnabledMap result;
    for (auto it = begin(); it != end(); ++it)
        result[it->index] = *it;
    return result;
}

DeviceIndexSet DeviceIndexEnabledMap::getEnabled() const
{
    DeviceIndexSet result;
    for (auto it = cbegin(); it != cend(); ++it)
        if(it->enabled)
            result << it.key();
    return result;
}
