/*
**    Copyright 2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef RCRUNID_H
#define RCRUNID_H

#include <QHash>
#include <QString>
#include <QMetaType>
#include <QUuid>

class QDebug;

struct RcRunId
{
    RcRunId() = default;
    RcRunId(const QString &index, int number, const QUuid &_uuid=QUuid()) :
        runIndex(index), runNumber(number), uuid(_uuid) {}

    [[deprecated("Use operator=()")]]
    void set(const RcRunId &r) { *this = r; }
    [[deprecated("Use = RcRunId()")]]
    void reset() { *this = RcRunId(); }
    void resetRunNumber() { runNumber = 0; }
    bool isValid() const;

    QString toString() const;
    [[deprecated("Use explicit formatting")]]
    explicit operator QString() const {
        return QString("%1 %2").arg(runIndex).arg(runNumber);
    }

    friend bool operator==(const RcRunId &l, const RcRunId &r)
    {
        return l.runIndex == r.runIndex
                && l.runNumber == r.runNumber
                && l.uuid == r.uuid;
    }
    friend bool operator!=(const RcRunId &l, const RcRunId &r) { return !(l == r); }
    friend bool operator<(const RcRunId &l, const RcRunId &r)
    {
        if (l.runIndex == r.runIndex)
            return l.runNumber < r.runNumber;
        return l.runIndex < r.runIndex;
    }
    friend uint qHash(const RcRunId &r, uint seed = 0) {
        return qHash(r.runIndex, seed) ^
                qHash(r.runNumber, seed) ^
                qHash(r.uuid, seed);
    }

    friend QDataStream &operator<<(QDataStream &out, const RcRunId &value);
    friend QDataStream &operator>>(QDataStream &in, RcRunId &value);
    friend QDebug operator<<(QDebug dbg, const RcRunId &r);

    QString runIndex;
    int runNumber = 0;
    QUuid uuid;
};

Q_DECLARE_METATYPE(RcRunId)


#endif // RCRUNID_H
