#ifndef BMCSTATUS_H
#define BMCSTATUS_H

#include <QMap>
#include <QString>

#include "base/fw_version.h"
enum DeviceId : uint8_t;

typedef enum {
    SENSOR_UNKNOWN  = 0,
    SENSOR_NORMAL   = 1, // numeric value used
    SENSOR_WARNING  = 2,
    SENSOR_CRITICAL = 3
} SensorStatus;

struct PowerSensor
{
    friend bool operator==(const PowerSensor &, const PowerSensor &) = default;

//    QString name;
    quint16 vol;
    quint16 cur;
};

struct BmcPower
{
    void clear() {
        *this = {};
    }
    QString getSummary(bool withCurrent = true, QString filter = {}) const;
    bool isCriticalCurrent() const;

    friend bool operator==(const BmcPower &, const BmcPower &) = default;

    QMap<QString, PowerSensor> sensors;
    bool setupInProgress = false;
};

struct BmcStatus
{
    BmcStatus() = default;
    explicit BmcStatus(const DeviceId &deviceId,
                       const fw_version_t &pcb_ver,
                       const QMap<quint16, quint16> &regs);

    bool isValid() const { return !mcuVer.isNull(); }
    QString get_overall_status_str() const { return get_status_str(overall_status); }
    QString get_power_status_str() const { return get_status_str(power_status); }
    QString get_pll_status_str() const { return get_status_str(pll_status); }
    inline bool onboard_pll_ok() const {return pll_locked; }
    //    return onboard_pll_status.b.locked; // inverted in TDC BMC before 2.2-94
    //}
    QString getVerStr() const { return mcuVer.toString(); }
    static QString get_status_str(SensorStatus st) {
        switch (st) {
        case SENSOR_UNKNOWN: return "unknown";
        case SENSOR_NORMAL: return "normal";
        case SENSOR_WARNING: return "warning";
        case SENSOR_CRITICAL: return "critical";
        }
        return {};
    }

    quint16 getPllUnlockCnt() const { return pllUnlockCnt; }
    BmcPower getPower() const { return power; }
    SensorStatus get_overall_status() const { return overall_status; }
    SensorStatus get_power_status() const { return power_status; }
    SensorStatus get_pll_status() const { return pll_status; }
    QMap<QString, double> getTemps() const { return temp; }
    QMap<QString, SensorStatus> getSubsystemStatus() const;

    friend bool operator==(const BmcStatus &, const BmcStatus &) = default;

private:
    int tempCount() const;
    QString tempLabel(int index) const;
    int powerCount() const;
    QString powerLabel(int index) const;

    DeviceId deviceId = {};
    fw_version_t pcb_ver;
    bool pll_locked = false;
    quint16 pll_source = 0;
    quint16 pllUnlockCnt = 0;
    SensorStatus overall_status = SENSOR_UNKNOWN;
    SensorStatus power_status = SENSOR_UNKNOWN;
    SensorStatus pll_status = SENSOR_UNKNOWN;

    QMap<QString, double> temp;
    BmcPower power;

    fw_version_t mcuVer;
//    quint16 onboard_pll_status = 0;
};

#endif // BMCSTATUS_H
