#ifndef CALRESULT_H
#define CALRESULT_H

#include <QMetaType>
#include <QMap>

#include "base/DeviceIndex.h"

using DacCalResult = QMap<int, double>;

class BaselineStats
{
public:
    friend bool operator==(const BaselineStats &l, const BaselineStats &r) = default;
    bool empty() const { return sampleCount == 0; }
    double peakFactor() const { return (max-min) / stddev; };
    QString toString() const;

    int sampleCount = 0;
    double min = 0;
    double max = 0;
    double mean = 0;
    double stddev = 0;
    double median = 0;
};
Q_DECLARE_METATYPE(BaselineStats)

class MultichannelBaselineStats : public QMap<int, BaselineStats>
{
public:
    explicit MultichannelBaselineStats(const QMap<int, QVector<int>> &allChannelData = {});
    QMap<int, double> meanMap() const;
    QMap<int, double> rmsMap() const;
    QString toString() const;
};
Q_DECLARE_METATYPE(MultichannelBaselineStats)

class CalResult
{
public:
    friend bool operator==(const CalResult &l, const CalResult &r) = default;

    DeviceIndex index;
    int nch=0;
    QMap<QString, double> temps;
    MultichannelBaselineStats baseline_x1;
    MultichannelBaselineStats baseline_x4;
    DacCalResult dacCal_mV;
};
Q_DECLARE_METATYPE(CalResult)

#endif // CALRESULT_H
