/*
**    Copyright 2009-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "FitterLinearLsq.h"
#include <cstdio>
#include <cmath>

namespace dsplib {

static inline double sqr(double x) { return pow(x, 2); }

/// Linear Least Squares fit
void FitterLinearLsq::LinearFit (const double *x, const double *y, int n, double *a, double *b, double *c)
{
    // Find a,b for array of (x,y) of length n
    // y = ax + b

    int i;
    double xa = 0, ya = 0, stt = 0, xy = 0, aa, bb, chi2 = 0;

    if (n < 1)
        return;

    for (i = 0; i < n; i++) {
        xa += x[i];
        ya += y[i];
        //    printf("%d: %f, %f\n", i, x[i], y[i]);
    }
    xa /= n;
    ya /= n;
    for (i = 0; i < n; i++)
        stt += sqr (x[i] - xa);
    for (i = 0; i < n; i++)
        xy += (x[i] - xa) * y[i];
    if (stt != 0)
        bb = xy / stt;
    else
        bb = 0;
    aa = ya - xa * bb;
    for (i = 0; i < n; i++)
        chi2 += sqr (y[i] - aa - bb * x[i]);
    //  printf("Line fit: a=%f, b=%f, chi2=%f\n", aa, bb, chi2);
    if (a)
        *a = bb;
    if (b)
        *b = aa;
    if (c)
        *c = chi2;
}

void FitterLinearLsq::LinearFitInt (const int *y, int n, double *a, double *b, double *c)
{
    // Find a,b for array of (x,y) of length n
    // y = ax + b

    int i;
    double ya = 0, stt = 0, xy = 0, aa, bb, chi2 = 0;

    if (n < 1)
        return;

    for (i = 0; i < n; i++) {
        ya += y[i];
    }
    ya /= n;
    for (i = 0; i < n; i++)
        stt += sqr (i - 1);
    for (i = 0; i < n; i++)
        xy += (i - 1) * y[i];
    if (stt != 0)
        bb = xy / stt;
    else
        bb = 0;
    aa = ya - bb;
    for (i = 0; i < n; i++)
        chi2 += sqr (y[i] - aa - bb * i);
    printf ("Line fit int: a=%f, b=%f, chi2=%f\n", aa, bb, chi2);
    if (a)
        *a = bb;
    if (b)
        *b = aa;
    if (c)
        *c = chi2;
}

} // namespace dsplib
