#include "AdcDeviceMgrFsm.h"

#include <QDebug>
#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.adc_device")
}


AdcDeviceMgrFsm::AdcDeviceMgrFsm(QObject *p):
    DeviceMgrFsm(p)
{
    setObjectName(__FUNCTION__);
    recreateInput(new AdcDeviceMgrFsmInput());
}

void AdcDeviceMgrFsm::setMainCalKey(const ConfigKey &key)
{
    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << key;
    getInput()->cal_key = key;
    evaluate();
}

void AdcDeviceMgrFsm::setDevCalKey(const DeviceIndex &index, const ConfigKey &key)
{
    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << index << key;
    getInput()->deviceCalKeys[index] = key;
    evaluate();
}

bool AdcDeviceMgrFsm::evaluate()
{
    if(DeviceMgrFsm::evaluate())
        return true;

    switch (getState()) {
    case FsmState::Ready:
        if (allDevicesWritten()) {
            if (getRequestState() == FsmState::Calibrate
                    || getRequestState() == FsmState::FindAdcDelay) {
                acceptRequestedState();
                return true;
            }
        }
        break;
    case FsmState::Calibrate:
        if(getRequestState() == FsmState::Ready
                && allDevicesCalibrated()) {
            acceptRequestedState();
            return true;
        }
        break;
    case FsmState::FindAdcDelay:
        if(getRequestState() == FsmState::Ready
                && allDevicesCalibrated()) {
            acceptRequestedState();
            return true;
        }
        break;
    default:;
    }

    return false;
}

void AdcDeviceMgrFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

qint64 AdcDeviceMgrFsm::stateTimeoutMs(const FsmState &state) const
{
    switch (state) {
    case FsmState::FindAdcDelay: return 15*60*1e3; // 15 min
    case FsmState::Calibrate: return 10e3;
    default:
        return DeviceMgrFsm::stateTimeoutMs(state);
    }
}

bool AdcDeviceMgrFsm::allDevicesCalibrated() const
{
    return checkDevKey(getInput()->cal_key, getInput()->deviceCalKeys);
}

QDebug operator<<(QDebug dbg, const AdcDeviceMgrFsmInput &in)
{
    dbg << DeviceMgrFsmInput(in);

    dbg << "deviceCalKeys:" << in.deviceCalKeys << "; ";
    dbg << "cal_key:" << in.cal_key << "; ";

    return dbg;
}
