#include "MStreamDeviceMgrFsm.h"

#include <QDebug>
#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.mstream_dev")
}

MStreamDeviceMgrFsm::MStreamDeviceMgrFsm(QObject *parent)
    : DeviceMgrFsm(parent)
{
    setObjectName(__func__);
    recreateInput(new MStreamDeviceMgrFsmInput());
}

bool MStreamDeviceMgrFsm::isTransitionAllowed(const FsmState &toState) const
{
    switch (getState()) {
    case FsmState::Ready:
        if(toState == FsmState::Configure) return allDevicesWritten();
        break;
    case FsmState::Configure:
        if(toState == FsmState::Starting) return allMstreamReseted();
        break;
    case FsmState::Run:
        if(toState == FsmState::Stopping) return true;
        break;
    default:;
    }
    return DeviceMgrFsm::isTransitionAllowed(toState);
}

void MStreamDeviceMgrFsm::setMStreamSeqNum(int seq)
{
    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << seq;
    getInput()->mstreamSeqNum = seq;
    evaluate();
}

void MStreamDeviceMgrFsm::setMStreamDevSet(const DeviceIndexSet &newSet)
{
    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << newSet;
    getInput()->mstreamSet = newSet;
    evaluate();
}

void MStreamDeviceMgrFsm::setDevMstreamSeqNum(const DeviceIndex &index, int seq)
{
    auto &keys = getInput()->mstreamSeqNumbers;
    auto it = keys.find(index);
    if(it == keys.end())
        keys[index] = seq;
    else if(*it != seq)
        *it = seq;
    else
        return;

    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << index << seq;
    evaluate();
}

bool MStreamDeviceMgrFsm::evaluate()
{
    if(DeviceMgrFsm::evaluate())
        return true;

    switch (getState()) {
    case FsmState::Run:
        if (!allMstreamReseted()) {
            qDebug(fsm) << objectName() << "Go to Error state (some mstream was reseted in run)";
            setState(FsmState::Error); // stop on error
            return true;
        }
        break;
    default:;
    }

    return false;
}

void MStreamDeviceMgrFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

qint64 MStreamDeviceMgrFsm::stateTimeoutMs(const FsmState &state) const
{
    switch (state) {
    case FsmState::Configure:
        return 1e3;
    default:
        return DeviceMgrFsm::stateTimeoutMs(state);
    }
}

bool MStreamDeviceMgrFsm::allMstreamReseted() const
{
    auto i = getInput();
    for(auto key : i->mstreamSet)
        if(i->mstreamSeqNumbers.value(key, 0) != i->mstreamSeqNum)
            return false;
    return true;
}

QDebug operator<<(QDebug dbg, const MStreamDeviceMgrFsmInput &in)
{
    dbg << DeviceMgrFsmInput(in);

    dbg << "mstreamSeqNum:" << in.mstreamSeqNum << "; ";
    dbg << "mstreamSet:" << in.mstreamSet << "; ";
    dbg << "mstreamSeqNumbers:" << in.mstreamSeqNumbers << "; ";

    return dbg;
}
