/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef FIRDESIGNCONFIG_H
#define FIRDESIGNCONFIG_H

#include "dsplib/WindowFunction.h"

class FirDesignConfig
{
public:
    using fwinf_t = dsplib::WindowFunction::fwinf_t;

    friend bool operator==(const FirDesignConfig &, const FirDesignConfig &) = default;

    int adc_rawdata_maxval = 0x7FFF;
    double latency = 0;
    double snf = -120;     //!< shape pulse threshold
    double pnf = -120;     //!< ref pulse threshold
    double ppos = 0;       //!< ref pulse position
    double pwid = 3;       //!< ref pulse width (2.35*sigma)
    int expfall = 10000;       //!< exponential falloff correction for shape reconstruction
    fwinf_t fwinf = dsplib::WindowFunction::WF_RECT;         //!< Filter window function
};

#endif // FIRDESIGNCONFIG_H
