#include "DeviceDynamicStateFilterProxyModel.h"

#include "BaseDeviceTableModel.h"

DeviceDynamicStateFilterProxyModel::DeviceDynamicStateFilterProxyModel(QObject *parent)
    : QSortFilterProxyModel(parent)
    , enabled(false)
    , show_dynamic(false)
{
}

bool DeviceDynamicStateFilterProxyModel::filterAcceptsRow(
        int sourceRow, const QModelIndex &sourceParent) const
{
    if(!enabled)
        return true;
    auto sourceModel = this->sourceModel();
    Q_ASSERT(sourceModel);
    if (!sourceModel)
        return true;
    const QModelIndex &indexStaticState = sourceModel->index(
                sourceRow, BaseDeviceTableModel::COL_DYNAMIC, sourceParent);

    indexStaticState.internalPointer();
    const auto &dynamicFlag = sourceModel->data(indexStaticState)
            .value<DeviceTableEntry::DynamicFlag>();
    const bool &isStatic = dynamicFlag == DeviceTableEntry::Static;

    if(show_dynamic) {
        const auto &indexOnline = indexStaticState.siblingAtColumn(
                    BaseDeviceTableModel::COL_ONLINE);
        const auto &onlineFlag = sourceModel->data(indexOnline, Qt::UserRole)
                .toBool();
        return !isStatic && onlineFlag;
    } else
        return isStatic;
}
