#include "SubnetListWidget.h"

#include "Ipv4PrefixFactory.h"
#include "mldiscover/LldpDiscovery.h"

SubnetListWidget::SubnetListWidget(QWidget *parent) : QListWidget(parent)
{
    setSelectionMode(QListWidget::ExtendedSelection);
    fillKnownNetworks();
}

void SubnetListWidget::lldpCacheUpdated(const LldpInfoCache &cache)
{
    for(auto &lldp : cache) {
        if(!knownSubnets.match(lldp.common.ip_addr)) {
            addSubnet(Ipv4Prefix(lldp.common.ip_addr, 24));
        }
    }
}

void SubnetListWidget::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected)
{
    QListWidget::selectionChanged(selected, deselected);

    Ipv4PrefixList subnetSet;
    const auto &indices = selectedIndexes();
    for (auto it = indices.begin(); it != indices.end(); ++it) {
        const auto &s = it->data(Qt::UserRole).toString();
        subnetSet.insert(Ipv4Prefix(s));
    }
    emit setNetworkFilter(subnetSet);
}

void SubnetListWidget::fillKnownNetworks()
{
    const auto &map = Ipv4PrefixFactory::create();
    for (auto it = map.begin(); it != map.end(); ++it) {
        addSubnet(it.key(), it.value());
    }
}

void SubnetListWidget::addSubnet(const Ipv4Prefix &subnet, QString name)
{
    if(knownSubnets.contains(subnet))
        return;
    knownSubnets.insert(subnet);
    int row = count();
    QString toolTip = subnet;
    if(name.isNull())
        name = toolTip;
    QListWidgetItem *item = new QListWidgetItem(name);
    item->setData(Qt::UserRole, QString(subnet));
    item->setToolTip(toolTip);
    insertItem(row, item);
    item->setSelected(true);
}
