/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "AbstractEasyDeviceModule.h"

#include "regio/regop.h"

AbstractEasyDeviceModule::AbstractEasyDeviceModule(const MregBusInterface &bus)
    : AbstractDeviceModule(bus)
{}

mlink::RegOpVector AbstractEasyDeviceModule::regOpRebase(const mlink::RegOpVector &data, quint16 baseAddr)
{
    mlink::RegOpVector result(data);
    result.rebase(baseAddr);
    return result;
}

bool AbstractEasyDeviceModule::regOpExecRebased(mlink::RegOpVector &data)
{
    data.rebase(baseAddr());
    return regOpExec(data);
}

bool AbstractEasyDeviceModule::regRead16(quint16 relAddr, quint16 *data)
{
    bool ok = false;
    RegOpVector r;
    r.RegRead16(relAddr, data, &ok);
    regOpExecRebased(r);
    return ok;
}

bool AbstractEasyDeviceModule::regRead32(quint16 relAddr, quint32 *data)
{
    bool ok = false;
    RegOpVector r;
    r.RegRead32(relAddr, data, &ok);
    regOpExecRebased(r);
    return ok;
}

bool AbstractEasyDeviceModule::regRead64(quint16 relAddr, quint64 *data)
{
    bool ok = false;
    RegOpVector r;
    r.RegRead64(relAddr, data, &ok);
    regOpExecRebased(r);
    return ok;
}

bool AbstractEasyDeviceModule::regWrite16(quint16 relAddr, quint16 data)
{
    bool ok = false;
    RegOpVector r;
    r.RegWrite16(relAddr, data, &ok);
    regOpExecRebased(r);
    return ok;
}

bool AbstractEasyDeviceModule::regWrite32(quint16 relAddr, quint32 data)
{
    bool ok = false;
    RegOpVector r;
    r.RegWrite32(relAddr, data, &ok);
    regOpExecRebased(r);
    return ok;
}

bool AbstractEasyDeviceModule::regWrite64(quint16 relAddr, quint64 data)
{
    bool ok = false;
    RegOpVector r;
    r.RegWrite64(relAddr, data, &ok);
    regOpExecRebased(r);
    return ok;
}
