/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "Ad5622ConfigConverter.h"
#include "Ad5622/Ad5622Config.h"

constexpr const char *k_boardSet = "boardSet";
constexpr const char *k_baseline = "baseline";
constexpr const char *k_level = "level";

template<>
bool AbstractConfigConverter<Ad5622Config>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<Ad5622Config>::toJsonObject(const Ad5622Config &in)
{
    QVariantMap m;
    m[k_boardSet] = QVariant::fromValue(in.boardSet);
    m[k_baseline] = QVariant::fromValue(in.baseline);
    m[k_level] = QVariant::fromValue(in.level);
    return fromVariantMap(m);
}

template<>
Ad5622Config AbstractConfigConverter<Ad5622Config>::fromJsonObject(const QJsonObject &in)
{
    Ad5622Config out;
    update_value(in, k_boardSet, &out.boardSet);
    update_value(in, k_baseline, &out.baseline);
    update_value(in, k_level, &out.level);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<Ad5622Config>::toJsonValue(const Ad5622Config &in)
{
    return toJsonObject(in);
}

template<>
Ad5622Config AbstractConfigConverter<Ad5622Config>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<Ad5622Config>::registerSubTypes()
{
}
