/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef CLOCK_STATUS_H
#define CLOCK_STATUS_H

#include <QtCore>

#include "ClockControlType.h"

typedef union {
    quint16 all;
    struct {
        quint32 pll_bypass: 1; // [0]
        quint32 unused: 15;    // [15:1]
    } bit;
} reg_clock_control;

typedef union {
    quint16 all;
    struct {
        quint32 logic_1: 1; // [0]
        quint32 ttc_locked: 1; // [1]
        quint32 vxs_locked: 1; // [2]
        quint32 unused: 13;    // [15:3]
    } bit;
} reg_clock_status;

typedef union {
    quint16 all;
    struct {
        quint32 osc: 1; // [0]
        quint32 ttc: 1; // [1]
        quint32 vxs: 1; // [2]
        quint32 unused: 13; // [15:3]
    } bit;
} reg_clock_source;

struct ClockStatus
{
    enum class Clock { local, TTC, VXS };
    friend bool operator==(const ClockStatus &l, const ClockStatus &r) = default;
    QString getSelectedClockStr() const;

    bool pll_bypass = false;
    QMap<ClockControlType, bool> clock_valid;
    ClockControlType selected_clock = ClockControlLocal;
};
Q_DECLARE_METATYPE(ClockStatus)

struct ClockBuildParams
{
    friend bool operator==(const ClockBuildParams &l, const ClockBuildParams &r) = default;

    quint16 clockCapability;
};
Q_DECLARE_METATYPE(ClockBuildParams)

#endif // CLOCK_STATUS_H
