#ifndef DESERIALIZERSCONTROLMODULE_H
#define DESERIALIZERSCONTROLMODULE_H

#include "device-modules/BaseDeviceModule.h"

#include <optional>

#include "DeserializersControlBuildParams.h"
#include "DeserializersControlStatus.h"

class AdcSerDesDelayMap;

class DeserializersControlModule : public BaseDeviceModule<DeserializersControlModule>
{
public:
    explicit DeserializersControlModule(const MregBusInterface &bus);

    static QList<AFI_SDB_ID> getSupportedSdbIds();

    void post_init() override;
    std::optional<DeserializersControlBuildParams> getBuildParams() const { return buildParams; }
    std::optional<DeserializersControlStatus> readStatus();
    bool setDeserReset(bool en);
    bool ResetAdcPin();
    bool setAdcSync();
    bool ResetDeser();

    bool ResetAdcStatus();
    bool setClockDelay(quint16 clkDelay);
    bool setDataDelay(quint16 dataDelay);
    bool setSerDesDelay(const AdcSerDesDelayMap &delay);

    bool ResetAdcClkDelay();

private:
    mlink::RegOpVector getClockDelaySetRegOp(quint16 clockDelay) const;
    mlink::RegOpVector getDataDelaySetRegOp(quint16 dataDelay) const;
    mlink::RegOpVector getStatusRegOp(quint16 *result) const;
    bool changeDesDelay(quint16 channel_mask, quint16 taps);
    bool changeDesDelay(const AdcSerDesDelayMap &delay);
    void readBuildParams();
    bool updateDesCtrlVal();
    bool pulseCtrlBit(quint16 bit);

    std::optional<DeserializersControlBuildParams> buildParams;

    quint16 curDesCtrlVal = 0;
};

#endif // DESERIALIZERSCONTROLMODULE_H
