/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef NETWORKPORTMODULEPRIVATE_H
#define NETWORKPORTMODULEPRIVATE_H

#include "NetworkPortConfig.h"
#include "NetworkPortStatus.h"

class NetworkPortModule;
namespace mlink { class RegOpVector; }

class NetworkPortModulePrivate
{
public:
    NetworkPortModulePrivate(NetworkPortModule *q)
        : q_ptr(q)
    {
        qRegisterMetaType<QMap<int, NetworkPortStatus> >();
    }

    template<typename T>
    bool isAvailable() const;

    template<typename T>
    mlink::RegOpVector getRegOp() const;

    NetworkPortDescription getDescription(const mlink::RegOpVector &regRes) const;
    NetworkPortDescription getDescription_v1() const;
    NetworkPortStatus getStatus(const mlink::RegOpVector &regRes) const;
    bool setConfig(const NetworkPortConfig &newConfig);

private:
    Q_DECLARE_PUBLIC(NetworkPortModule);
    NetworkPortModule * const q_ptr;

    NetworkPortDescription descr;
    NetworkPortConfig config;
};

#endif // NETWORKPORTMODULEPRIVATE_H
