#include "PllSi5326Module.h"

#include <QElapsedTimer>
#include <QThread>

#include "base/DeviceId.h"
#include "mregdev/qtmregdevice.h"
#include "regio/regop.h"

enum {
    REG_CMD_BASE        = 0x00,
    REG_FSM_CRTL        = 0x80,
    REG_FSM_STATE       = 0x81,
    REG_COMMANDS_NUM    = 0x82,
    REG_STATUS          = 0x83,
};

enum {
    BIT_CTRL_IDLE               = 0x00,
    BIT_CTRL_RECONFIG_REQUEST   = 0x01, // PLL reconfig request (start SPI sequence)
    BIT_CTRL_RAM_ACCESS_REQUEST = 0x02, // request to communicate with configuration RAM
};
enum {
    BIT_FSM_INIT            = 0x01,
    BIT_FSM_IDLE            = 0x02,
    BIT_FSM_SOFTWARE_ACCESS = 0x04, // software RAM read/write
    BIT_FSM_READ_RAM_CMD    = 0x08, // read command from RAM into SPI PHY
    BIT_FSM_SPI_ADDR_WR     = 0x10, // SPI PHY address write
    BIT_FSM_SPI_DATA_WR     = 0x20  // SPI PHY data write
};

enum {
    BIT_STATUS_LOL      = 0x1, // PLL LOL pin
    BIT_STATUS_INT_C1B  = 0x2, // PLL INT_C1B pin
    BIT_STATUS_C2B      = 0x4 // PLL C2B pin
};

PllSi5326Module::PllSi5326Module(const MregBusInterface &bus)
    : BaseDeviceModule(bus)
{
    if (!isAvailable()) {
        if (getDeviceId() == DEVICE_ID_PHOS_FEC)
            setBaseAddr(0x4400, 0xFF);
    }
}

QList<AFI_SDB_ID> PllSi5326Module::getSupportedSdbIds()
{
    return {};
}

void PllSi5326Module::post_init()
{
    if (getDeviceId() == DEVICE_ID_PHOS_FEC) {
//        patchPllConfig();
    }
}

bool PllSi5326Module::waitFsmState(quint16 state)
{
    QElapsedTimer timer;
    timer.start();
    while(timer.elapsed() < 1e3) {
        quint16 data;
        RegOpVector r;
        r.RegRead16(REG_FSM_STATE, &data);
        if (!regOpExecRebased(r))
            return false;
        if (data == state)
            return true;
    }
    qWarning("%s failed on timeout while waiting state: 0x%xd", __PRETTY_FUNCTION__, state);
    return false;
}

bool PllSi5326Module::patchPllConfig()
{
    if (!regWrite16(baseAddr() + REG_FSM_CRTL, BIT_CTRL_RAM_ACCESS_REQUEST))
        return {};
    if(!waitFsmState(BIT_FSM_SOFTWARE_ACCESS)) {
        return {};
    }
    constexpr static const quint16 PLL_COMMAND_2 = 0x242;
    quint16 readval;
    RegOpVector r;
    r.RegWrite16(REG_CMD_BASE + 2, PLL_COMMAND_2); // cmd#3 0x2a2 -> 0x242
    r.RegRead16 (REG_CMD_BASE + 2, &readval);
    r.RegWrite16(REG_FSM_CRTL, 0);
    r.RegWrite16(REG_FSM_CRTL, BIT_CTRL_RECONFIG_REQUEST);
    r.RegWrite16(REG_FSM_CRTL, 0);

    if (!regOpExecRebased(r))
        return {};

    if (readval != PLL_COMMAND_2)
        qCritical("Failed to write new PLL command to RAM regVal:0x%xd (expected 0x%xd)",
                  readval, PLL_COMMAND_2);

    if(!waitFsmState(BIT_FSM_IDLE)) {
        return {};
    }

    // Wait end of PLL internal calibration (the last PLL SPI command)
    QThread::sleep(1);
    return true;
}
