#ifndef RUNLOGICSTATUS_H
#define RUNLOGICSTATUS_H

#include <QMetaType>

class TaiTime;

class RunLogicStatus
{
public:
    enum class State { Idle, Running, Finished };
    friend bool operator==(const RunLogicStatus &, const RunLogicStatus &) = default;
    RunLogicStatus& operator-=(const RunLogicStatus& r);
    friend RunLogicStatus operator-(RunLogicStatus l, const RunLogicStatus &r)
    {
        l -= r;  return l;
    }

    bool isValid() const;
    bool isIdle() const { return getState() == State::Idle; }
    bool isRunning() const { return getState() == State::Running; }
    bool hasFinished() const { return getState() == State::Finished; }

    TaiTime startTime() const;
    TaiTime finishTime() const;
    TaiTime currentTime() const;

    State getState() const;
    QString getStateStr() const;
    QString getFullText() const;

    quint32 id = 0;
    quint16 state = 0;
    quint32 currentRunTime = 0;
    quint32 currentEventCount = 0;
    quint64 currentTimeRaw = 0;
    quint64 startTimeRaw = 0;
    quint64 finishTimeRaw = 0;
};
Q_DECLARE_METATYPE(RunLogicStatus)

#endif // RUNLOGICSTATUS_H
