#include "Tlu40LvdsLuminosityModule.h"

#include "DeviceModuleCommand.h"
#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

namespace {
enum {
    REG32_ID                = 0x00,
    REG_CTRL                = 0x02, // [0] - reset
    REG_LOCK                = 0x03, // [0] - 1: lock, 0: continuous update
    REG_NUM_CH              = 0x04, // Number of channels, RO static
    REG32_CLK_FREQ          = 0x05, // Clock frequency, kHz, 32 bits, RO static
    REG64_CH_EN             = 0x10, // Channel enable, 64 bits, R/W
    REG_SHAPER_VAL          = 0x14, // Shaper value, R/W
    REG64_CH_CUR_INPUT_ST   = 0x18, // Channels current input state, RO
    REG32_CH_INPUT_CNT_BASE = 0x100, // 100h~107h - Input counters, 32 bits per channel, RO
};
}

QList<AFI_SDB_ID> Tlu40LvdsLuminosityModule::getSupportedSdbIds()
{
    return {AFI_SDB_TLU40LVDS_Luminosity};
}

bool Tlu40LvdsLuminosityModule::writeConfig(const Tlu40LvdsLuminosityConfig &config)
{
    if(!buildParams)
        return {};

    quint64 chEn = 0;
    for(auto ch=0; ch<buildParams->chNum; ++ch)
        if(config.chEn.value(ch, false))
            chEn |= 1<<ch;

    RegOpVector r;
    r.RegWrite16(REG_SHAPER_VAL, config.shaperVal);
    r.RegWrite64(REG64_CH_EN, chEn);
    return regOpExecRebased(r);
}

std::optional<Tlu40LvdsLuminosityStatus> Tlu40LvdsLuminosityModule::readStatus()
{
    if(!buildParams)
        return {};

    Tlu40LvdsLuminosityStatus st;
    st.chCnt.resize(buildParams->chNum);

    RegOpVector r;
    r.RegWrite16(REG_LOCK, 0x1);
    r.RegRead64(REG64_CH_CUR_INPUT_ST, &st.inputSt);
    for(auto ch=0; ch<buildParams->chNum; ++ch)
        r.RegRead32(REG32_CH_INPUT_CNT_BASE+ch*2, &st.chCnt[ch]);
    r.RegWrite16(REG_LOCK, 0x0);

    if(!regOpExecRebased(r))
        return {};
    return st;
}

void Tlu40LvdsLuminosityModule::post_init()
{
    buildParams.reset();
    quint32 id;
    Tlu40LvdsLuminosityBuildParams bp;

    RegOpVector r;
    r.RegRead32(REG32_ID, &id);
    r.RegRead16(REG_NUM_CH, &bp.chNum);
    r.RegRead32(REG32_CLK_FREQ, &bp.clkFreq);

    if(regOpExecRebased(r) && getSupportedSdbIds().contains(static_cast<AFI_SDB_ID>(id))) {
        buildParams = bp;
    }
 }
