/*
**    Copyright 2014-2016 Ivan Filippov
**    Copyright 2014-2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "PhysicalTriggerConfig.h"

#include <QDataStream>
#include <QDebug>


bool operator==(const PhysicalTriggerChConfig &l, const PhysicalTriggerChConfig &r)
{
    return l.name == r.name &&
            l.reg.all == r.reg.all;
}

QDebug operator<<(QDebug dbg, const PhysicalTriggerChConfig &s)
{
    dbg << "PhysicalTriggerChConfig{"
        << "name:" << s.name
        << "reg.all:" << hex << s.reg.all

        << "reg.bit.after_protect_time:" << s.reg.bit.after_protect_time
        << "reg.bit.before_protect_time:" << s.reg.bit.before_protect_time
        << "reg.bit.reduction_factor:" << s.reg.bit.reduction_factor
        << "reg.bit.delay:" << s.reg.bit.delay
        << "reg.bit.reduction_en:" << s.reg.bit.reduction_en
        << "reg.bit.after_protect_en:" << s.reg.bit.after_protect_en
        << "reg.bit.before_protect_en:" << s.reg.bit.before_protect_en
        << dec << "}";
    return dbg;
}

QDataStream &operator<<(QDataStream &out, const PhysicalTriggerChConfig &r)
{
    out
            << r.name
            << r.reg.all
               ;
    return out;
}

QDataStream &operator>>(QDataStream &in, PhysicalTriggerChConfig &r)
{
    in
            >> r.name
            >> r.reg.all
            ;
    return in;
}

bool operator==(const CalibrationTriggerConfig &l, const CalibrationTriggerConfig &r)
{
    return l.en == r.en &&
            l.period == r.period &&
            l.count == r.count &&
            l.offset == r.offset;
}

QDebug operator<<(QDebug dbg, const CalibrationTriggerConfig &s)
{
    dbg << "CalibrationTriggerConfig{"
        << "en:" << s.en
        << "period:" << s.period
        << "count:" << s.count
        << "offset:" << s.offset
        << "}";
    return dbg;
}

QDataStream &operator<<(QDataStream &out, const CalibrationTriggerConfig &r)
{
    out
            << r.en
            << r.period
            << r.count
            << r.offset
               ;
    return out;
}

QDataStream &operator>>(QDataStream &in, CalibrationTriggerConfig &r)
{
    in
            >> r.en
            >> r.period
            >> r.count
            >> r.offset
            ;
    return in;
}

bool operator==(const PhysicalTriggerConfig &l, const PhysicalTriggerConfig &r)
{
    return l.matchWin == r.matchWin &&
            l.trigDelay == r.trigDelay &&
            l.calibTrigger == r.calibTrigger &&
            l.startChMask == r.startChMask &&
            l.readChMask == r.readChMask &&
            l.chWfLatency == r.chWfLatency &&
            l.chWfReadPoints == r.chWfReadPoints &&
            l.oscEn == r.oscEn &&
            l.oscDelayed == r.oscDelayed &&
            l.chCfg == r.chCfg &&
            l.trig == r.trig &&
            l.spill == r.spill;
}

QDebug operator<<(QDebug dbg, const PhysicalTriggerConfig &s)
{
    dbg << "PhysicalTriggerConfig{"
        << "matchWin:" << s.matchWin
        << "trigDelay:" << s.trigDelay
        << "calibTrigger:" << s.calibTrigger
        << "startChMask:" << s.startChMask
        << "readChMask:" << s.readChMask
        << "chWfLatency:" << s.chWfLatency
        << "chWfReadPoints:" << s.chWfReadPoints
        << "oscEn:" << s.oscEn
        << "oscDelayed:" << s.oscDelayed
        << "chCfg:" << s.chCfg
        << "trig:" << s.trig
        << "spill:" << s.spill
        << "}";
    return dbg;
}

QDataStream &operator<<(QDataStream &out, const PhysicalTriggerConfig &r)
{
    out
            << r.matchWin
            << r.trigDelay
            << r.calibTrigger
            << r.startChMask
            << r.readChMask
            << r.chWfLatency
            << r.chWfReadPoints
            << r.oscEn
            << r.oscDelayed
            << r.chCfg
            << r.trig
            << r.spill;
    return out;
}

QDataStream &operator>>(QDataStream &in, PhysicalTriggerConfig &r)
{
    in
            >> r.matchWin
            >> r.trigDelay
            >> r.calibTrigger
            >> r.startChMask
            >> r.readChMask
            >> r.chWfLatency
            >> r.chWfReadPoints
            >> r.oscEn
            >> r.oscDelayed
            >> r.chCfg
            >> r.trig
            >> r.spill;
    return in;
}
