#include "WrTimeEmulatorModule.h"

#include "regio/regop.h"

#include "sdb/AfiSdbId.h"

#include <QDateTime>

namespace {
// Reg. map ver. 2.0
enum {
    REG32_ID          = 0x00,
    REG_COTROL        = 0x02,
    REG_LOCK          = 0x03,
    REG_LOAD_TAI_BASE = 0x04, // 3*16-bit
};
inline constexpr const quint16 REG_COTROL_BIT_UPDATE = 0x0002;

// Reg. map ver. 1.0
enum {
    REG_TAI_0       = 0x00, // tai sec 15:0
    REG_TAI_1       = 0x01, // tai sec 31:16
    REG_TAI_2       = 0x02, // tai sec 39:32
    REG_TAI_UPDATE  = 0x03, // any write for update
};
}

QList<AFI_SDB_ID> WrTimeEmulatorModule::getSupportedSdbIds()
{
    return { AFI_SDB_WR_TIME_EMULATOR };
}

void WrTimeEmulatorModule::post_init()
{
    auto sec = QDateTime::currentSecsSinceEpoch();
    RegOpVector regOp;
    quint16 taiBaseAddr;
    switch (majorVersion()) {
    case 1:
        taiBaseAddr = REG_TAI_0;
        break;
    case 2:
    {
        quint32 sdbId;
        if (!regRead32(REG32_ID, &sdbId) || !getSupportedSdbIds().contains(static_cast<AFI_SDB_ID>(sdbId)))
            return;

        regOp.RegWrite16(REG_COTROL, 0);
        taiBaseAddr = REG_LOAD_TAI_BASE;

        break;
    }
    default:
        return;
    }

    for(auto i=0; i<3; ++i) {
        regOp.RegWrite16(taiBaseAddr+i, sec & 0xFFFF);
        sec >>= 16;
    }

    switch (majorVersion()) {
    case 1:
        regOp.RegWrite16(REG_TAI_UPDATE, 0);
        break;
    case 2:
        regOp.RegWrite16(REG_COTROL, REG_COTROL_BIT_UPDATE);
        regOp.RegWrite16(REG_COTROL, 0);
        break;
    default:
        break;
    }

    regOpExecRebased(regOp);
}
