#include "AdcMpdTrigDialog.h"
#include "ui_AdcMpdTrigDialog.h"
#include "AdcMpdTrigModel.h"
#include "AdcMpdTrigPolarityDelegate.h"
#include "AdcMpdTrigSpinBoxDelegate.h"
#include "AdcMpdTrigColumns.h"

#include "base/Globals.h"

namespace {
static const int THR_COL = static_cast<int>(AdcMpdTrigColumns::COL_GROUP_THR);
static const int ADC_COL = static_cast<int>(AdcMpdTrigColumns::COL_ADC_CH);
static const int POL_COL = static_cast<int>(AdcMpdTrigColumns::COL_POL);
}

AdcMpdTrigDialog::AdcMpdTrigDialog(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::AdcMpdTrigDialog)
    , model(new AdcMpdTrigModel(this))
    , thrSbDelegate(new AdcMpdTrigSpinBoxDelegate(this))
    , adcSbDelegate(new AdcMpdTrigSpinBoxDelegate(this))
    , polDelegate(new AdcMpdTrigPolarityDelegate(this))
{
    ui->setupUi(this);
    ui->treeView->setModel(model);
    ui->treeView->setItemDelegateForColumn(THR_COL, thrSbDelegate);
    ui->treeView->setItemDelegateForColumn(ADC_COL, adcSbDelegate);
    ui->treeView->setItemDelegateForColumn(POL_COL, polDelegate);
    connect(model, &AdcMpdTrigModel::configChanged, this, &AdcMpdTrigDialog::configChanged);
    connect(model, &QAbstractItemModel::modelReset, this, &AdcMpdTrigDialog::onModelReset);

    ui->treeView->header()->setStretchLastSection(false);
    ui->treeView->header()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

AdcMpdTrigDialog::~AdcMpdTrigDialog()
{
    delete ui;
}

void AdcMpdTrigDialog::setDeviceList(const DeviceIndexEnabledMap &devices_)
{
    model->setDeviceSet(devices_.getEnabled());
}

void AdcMpdTrigDialog::updateRemoteBlocking(bool remoteConnected) const
{

}

void AdcMpdTrigDialog::setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig)
{
    // Q_UNUSED(config_key)
    if (appConfig == newConfig)
        return;

    appConfig = newConfig;
    model->setConfig(config_key, appConfig);
}

void AdcMpdTrigDialog::onFsmStateChange(const FsmState &state)
{
    bool en = state==FsmState::Ready || state==FsmState::Error
        || Globals::getDebugMode();
    // setEnabled(en);
    model->setEnabled(en);
}

void AdcMpdTrigDialog::onModelReset()
{
    addDelegate({});
    ui->treeView->expandAll();
    updateGeometry();

    for(auto c : {ADC_COL, THR_COL, POL_COL}) {
        ui->treeView->resizeColumnToContents(c);
    }
}

void AdcMpdTrigDialog::addDelegate(const QModelIndex &index)
{
    auto view = ui->treeView;
    for(auto c : {ADC_COL, THR_COL, POL_COL}) {
        auto sibIndex = index.siblingAtColumn(c);
        if(!sibIndex.isValid())
            continue;
        if(!view->isPersistentEditorOpen(sibIndex))
            view->openPersistentEditor(sibIndex);
    }

    int chRows = model->rowCount(index);
    for(int i=0; i<chRows; ++i) {
        auto chIndex = model->index(i, 0, index);
        addDelegate(chIndex);
    }

    // if(!index.isValid())
    //     ui->treeView->resizeColumnToContents(c);
}
