#ifndef ADCMPDTRIGMODEL_H
#define ADCMPDTRIGMODEL_H

#include <QAbstractItemModel>
#include <QModelIndex>
#include <QVariant>

#include "base/DeviceIndex.h"
#include "lib-common/BaseDeviceAppConfig.h"

#include "AdcMpdTrigModelItem.h"
#include "AdcMpdTrig/AdcMpdTrigBuildParams.h"
// #include "AdcMpdTrig/AdcMpdTrigStatus.h"

class QTimer;
class ConfigKey;

class AdcMpdTrigModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    AdcMpdTrigModel(QObject *parent = nullptr);

    QVariant data(const QModelIndex &index, int role) const override;
    virtual bool setData(const QModelIndex &index,
                         const QVariant &value,
                         int role = Qt::EditRole) override;
    Qt::ItemFlags flags(const QModelIndex &index) const override;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role) const override;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const override;
    QModelIndex parent(const QModelIndex &index) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    void setEnabled(bool en);
    int getDevRow(const DeviceIndex &di) const;

signals:
    void configChanged(const BaseDeviceAppConfig &config) const;

public slots:
    void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig);
    void setAdcMpdTrigBp(const DeviceIndex &index, const AdcMpdTrigBuildParams &bp);
    void setDeviceSet(const DeviceIndexSet &set);

private slots:
    void postCheckStateChange();

private:
    void enqueueConfigSend();
    void doReset();
    void sendDataChanged(QVector<int> roles = QVector<int>(), const QModelIndex parent = {});

    QVariant displayData(const QModelIndex &index) const;
    QVariant editData(const QModelIndex &index) const;
    QVariant checkStateData(const QModelIndex &index) const;
    QVariant limitsData(const QModelIndex &index) const;
    bool setEditRole(const QModelIndex &index, const QVariant &value);
    bool setCheckStateData(const QModelIndex &index, const QVariant &value);

    inline AdcMpdTrigModelItem *getItemByIndex(const QModelIndex &index) const;
    QVariant getRecursiveState(const QModelIndex &mIndex, bool AdcMpdTrigChConfig::*ptr) const;

    DeviceIndexSet deviceSet;
    BaseDeviceAppConfig config;
    // QMap<DeviceIndex, QMap<int, AdcMpdTrigStatus>> dev_status;
    QMap<DeviceIndex, AdcMpdTrigBuildParams> devBp;
    AdcMpdTrigModelItem rootItem;
    QTimer *const delayedSendConfigTimer;
    bool enable = true;
};

#endif // ADCMPDTRIGMODEL_H
