#ifndef HIST_H
#define HIST_H

#include <QMetaType>
#include <QString>
#include <QVector>
#include "HistMngrCfg.h"

struct DistributionParams {
    DistributionParams() : cnt(0), mean(0), rms(0), min(0), max(0) {}
    uint cnt;
    double mean;
    double rms;
    double min;
    double max;
};

struct Histogramm
{
    Histogramm();
    Histogramm(QString _title, int _nBins, double _min, double _max);
    bool isEmpty();
    void reset();
    void resize(int _nBins, double _min, double _max);
    void fill(double v, int w=1);
    int findBin(double v) const;
    uint getUnderFlow() const { return under_flow; }
    uint getOverFlow() const { return over_flow; }
    uint getEntries() const;
    double getBinLower(int bin) const;
    double getBinCenter(int bin) const;
    double getBinUpper(int bin) const;
    DistributionParams getDistribution() const;
    bool operator==(const Histogramm& hist)
    {
         return (title == hist.title &&
                 n_bins == hist.n_bins &&
                 min == hist.min &&
                 max == hist.max &&
                 bin_step == hist.bin_step &&
                 bins == hist.bins &&
                 under_flow == hist.under_flow &&
                 over_flow == hist.over_flow &&
                 n_entries == hist.n_entries &&
                 min_filled_val == hist.min_filled_val &&
                 max_filled_val == hist.max_filled_val
         );
    }

    QString title;
    int n_bins; // exclude under/over-flow
    double min;
    double max;
    double bin_step;

    QVector<double> bins;
    uint under_flow;
    uint over_flow;
    uint n_entries;
    double min_filled_val;
    double max_filled_val;
    int min_filled_bin;
    int max_filled_bin;
};
Q_DECLARE_METATYPE(Histogramm)
#endif // HIST_H
