#ifndef MPDDATADECODER_H
#define MPDDATADECODER_H

#include <QObject>
#include <QByteArray>

#include "lib-common/MpdRawTypes.h"

#include "DeviceData.h"
#include "AnalyzerStat.h"

struct RcRunId;

class MpdDataDecoder : public QObject
{
    Q_OBJECT
public:
    MpdDataDecoder(QObject *p);
    void reset();

    void requestRawData(int events);
    void setRawDataEveryEv(bool everyEv);
    void invertUpdated(QMap<DeviceIndex, QVector<bool> > _invertMap);
    void set_can_skip(bool skip);
    void prepareForStop(quint32 evNum);
    void startWrite(bool rawData);
    void stopWrite();
    void setSkipLogic(std::function<bool(void)> f) { skipFunc = f; }
    void setFlushFunc(std::function<void(void)> f) { flushFunc = f; }
    AnalyzerStat getAnalyzerStats() const { return analyzerStats; }
    DecodedData getDecodedData() const { return decodedData; }

public slots:
    void dataAccuired(const QByteArray &newData);

signals:
    void rawDataUpdated(const QStringList &data) const;
    void gotStopEvent(quint32 stopEvNum) const;
    void runStarted(const RcRunId &) const;
    void runStoped(const RcRunId &) const;

private:
    void read();
    bool checkSkip();
    void sendRunStartStop(bool start, const RcRunId &id);

    bool findNextRecord();
    bool readRecord();
    void readEventRec();
    void readDeviceEvent();
    quint32 readFileRunRec();
    quint32 readStatRec();
    DeviceIndex getCurDeviceIndex() const;

    void readJson();

    // print functions
    void printCurrentDevice() const;
    void printRawCurMpdHdr();
    void printRawMpdHdr(const MpdHdr * const h);
    void printRawCurEvNum();
    void printRawCurMpdDevHdr();
    void printRawCurTai();
    void printRawCurMStreamHdr(bool isPureAdcDev = false);
    void printRawTxtHeader(const QString &header);
    void printRawTxtBlock(const char *ptr, uint size, const QString &header, int byteShift=0);
    void printRawWord(const void *word, QString suffix=QString());
    void flushRawDataEvent();

    bool isPureAdcDev() const;
    bool isTadcDev() const;
    bool isTtvxsDev() const;
    bool isTrcDev() const;

    void decodeMStreamType0(); // readDeviceEvent
    void decodeTsTai(const void* ptr);
    void decodeTrcBlock(); // decodeMStreamType0
    void decodeModuleDataBlockTdc(const quint32* payloadPtr); // decodeMStreamType0
    void decodeModuleDataBlockAdc(const quint32* payloadPtr); // decodeMStreamType0
    void decodeModuleDataBlockTtvxs(const quint32* payloadPtr); // decodeMStreamType0
    void decodeModuleDataBlockStat(const quint32* payloadPtr); // decodeMStreamType0
    void decodeMStreamType1Adc(const quint32* payloadPtr); // readDeviceEvent
    void decodeMscBlock(const quint32* payloadPtr);

    void flushData();

    QByteArray data;
    uint curPos = 0;
    uint curEventStartPos = 0;

    const MpdHdr *mpdHdr = nullptr;
    const quint32 *evNumPtr = nullptr;
    const MpdDeviceHdr *mpdDeviceHdr = nullptr;
    const MpdMStreamHdr *mstreamHdr = nullptr;
    const TaiRawData *tsTAI = nullptr;
    const DataBlockHeader *dataBlockHdr = nullptr; // seems will be deprecated

//not decoder fields
    quint32 lastEventNumAtPrevFrame = 0;
    quint32 prevEventNum = 0;

    AnalyzerStat analyzerStats;
    DecodedData decodedData;

    CompleteEvent curCompleteEv;
    TqdcEvent curTqdcEv;
    TrcEvent curTrcEv;
    TtvxsEvent curTtvxsEv;
    DevStat curTdcStat;
    TdcChHit curTdcHit;
    AdcChHit curAdcHit;
    MscSliceData curMscSlice;
    bool signalInverted = false;
    bool writeDecodedEnabled = false;
    QStringList rawDataTxt;
    int rawDataEvents = 0; // number events that should be listed (tab rawData)
    bool printBadEvent = true;
    bool rawDataEveryEv = false;
    QStringList lastErrStr;
    int clear_err_stack = 0;
    QMap<DeviceIndex, QVector<bool> > invertMap; // invert TDC edges
    bool can_skip = true;
    bool lookForStop = false;
    quint32 stopEvNum = 0;
    const bool adcRawDataSigned = true;
    std::function<bool(void)> skipFunc = nullptr;
    std::function<void(void)> flushFunc = nullptr;
};

#endif // MPDDATADECODER_H
