#include "JsonTreeModel.h"

#include "JsonTreeItem.h"

#include <QJsonValue>
#include <QJsonObject>

JsonTreeModel::JsonTreeModel(QObject *parent)
    : QAbstractItemModel(parent)
    , mRootItem{new JsonTreeItem}
{
}

JsonTreeModel::~JsonTreeModel()
{
    delete mRootItem;
}

void JsonTreeModel::clear()
{
    beginResetModel();
    mRootItem->clear();
    endResetModel();
}

bool JsonTreeModel::loadJson(const QJsonObject &obj)
{
    beginResetModel();
    delete mRootItem;
    mRootItem = JsonTreeItem::createItem(QJsonValue(obj));
    mRootItem->setType(QJsonValue::Object);
    endResetModel();
    return true;
}

QVariant JsonTreeModel::data(const QModelIndex &index, int role) const
{
    if (role != Qt::DisplayRole)
        return QVariant();

    if (!index.isValid())
        return QVariant();

    JsonTreeItem *item = static_cast<JsonTreeItem*>(index.internalPointer());

    if (index.column() == JSON_MODEL_COL_KEY)
        return QString("%1").arg(item->key());
    if (index.column() == JSON_MODEL_COL_VAL)
        return QString("%1").arg(item->value());
    return QVariant();
}

QVariant JsonTreeModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role != Qt::DisplayRole)
        return QVariant();

    if (orientation == Qt::Horizontal) {
        switch(section){
        case JSON_MODEL_COL_KEY:
            return keyHeaderName;
        case JSON_MODEL_COL_VAL:
            return valueHeaderName;
        }
    }

    return QVariant();
}

QModelIndex JsonTreeModel::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();

    JsonTreeItem *parentItem;

    if (!parent.isValid())
        parentItem = mRootItem;
    else
        parentItem = static_cast<JsonTreeItem*>(parent.internalPointer());

    JsonTreeItem *childItem = parentItem->child(row);
    if (childItem)
        return createIndex(row, column, childItem);
    else
        return QModelIndex();
}

QModelIndex JsonTreeModel::parent(const QModelIndex &index) const
{
    if (!index.isValid())
        return QModelIndex();

    JsonTreeItem *childItem = static_cast<JsonTreeItem*>(index.internalPointer());
    JsonTreeItem *parentItem = childItem->parent();

    if (parentItem == mRootItem)
        return QModelIndex();

    return createIndex(parentItem->row(), 0, parentItem);
}

int JsonTreeModel::rowCount(const QModelIndex &parent) const
{
    JsonTreeItem *parentItem;
    if (!parent.isValid())
        parentItem = mRootItem;
    else
        parentItem = static_cast<JsonTreeItem*>(parent.internalPointer());

    return parentItem->childCount();
}

int JsonTreeModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    return showValueCoumn ? JSON_MODEL_COL_TOTAL : JSON_MODEL_COL_TOTAL-1;
}
