/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ReadoutAppConfigDb.h"
#include "base/AbstractConfigConverter.h"

ReadoutAppConfigDb::ReadoutAppConfigDb(QObject *parent):
    AbstractConfigDb(parent)
{
    AbstractConfigConverter<BaseDeviceAppConfig>::registerTypes();
}

void ReadoutAppConfigDb::setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig)
{
    Q_UNUSED(config_key);
    if(!initialized)
        return;
    if (newConfig == config)
        return;
    config = newConfig;

    auto json = QVariant::fromValue(config).value<QJsonObject>();
    write_config(json);
}

void ReadoutAppConfigDb::update_config_from_db()
{
    auto json = readConfig(cs);
    config = QVariant::fromValue(json).value<BaseDeviceAppConfig>();
    config.fixAfterDbRead();

    emit configChanged(config);
}
