//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef RCCLIENTMANAGERWIDGET_H
#define RCCLIENTMANAGERWIDGET_H

#include "../RunStates.h"
#include "../config/RcClientConfig.h"
#include "../config/RcClientProtoVersions.h"
#include "remote-control-server/types.h"
#include "../RcClientData.h"
#include "../RcRunInfo.h"
#include "fsm/FsmState.h"
#include "pnp-server/PNPMessage.h"
#include "mvc/ClientEnableTableModel.h"

#include <QWidget>
#include <QMap>
#include <QDateTime>
#include <QUuid>

class QHostAddress;
class Client;
class ClientManagerFsm;
class Config;
class PnpInfoCache;
class ProgramDescription;
class QElapsedTimer;
class RcClientManager;
class RcClientStatusModel;
class MergeColumnsTableModel;
class RcClientTreeModel;
struct RcProgramState;
class QItemSelection;

namespace Ui {
class RcClientManagerWidget;
}

class RcClientManagerWidget : public QWidget
{
    Q_OBJECT
public:
    explicit RcClientManagerWidget(QWidget *parent = nullptr);
    virtual ~RcClientManagerWidget() override;

signals:
    void deleteClient(const ClientIndex &index); // 0 - for all
    void startListen(const ClientIndex &index); // 0 - for all
    void stopListen(const ClientIndex &index); // 0 - for all
    void sendRunState(FsmState state, const RcRunId &id);
    void clientInfoUpdated(const ClientInfo &info) const; // proxy to Model
    void clientRcProgramStateUpdated(const RcProgramState &state); // proxy to Tree Model
    void myClientIndexUpdated(const ClientIndex &index) const; // proxy to Tree Model

signals: void rcClientsConfigChanged(const RcClientConfigMap &client_config) const;
public slots: void setRcClientsConfig(const RcClientConfigMap &client_config_);

public slots:
    void onFsmStateChange(const FsmState &state);
    void pnpCacheUpdated(const PnpInfoCache &cache);

private slots:
    void on_pushButtonAdd_clicked();
    void on_pushButtonRemove_clicked();
    void selectionChanged(const QItemSelection &selected);

    void updatePendingProgramDescription();
    void updateButtonsState();
    void onClientInfoUpdated(const ClientInfo &info); // proxy to Model
    void setClientIndexEnabledMap(const ClientIndexEnabledMap &map);
    void treeModelReseted();

private:
    void setupTable();
    int getClientRow(const ClientIndex &index) const;
    inline ClientIndex getClientId(int row) const;

    bool hasFreeProgs() const;
    bool isFreeProgs(const ProgramDescription &descr) const;
    ClientIndex getSelectedClient() const;
    void updateVisibilityColumn();
    void updateVisibilityColumn(int col);
    FsmState getExpectedClientsState(const FsmState &state) const;
    bool isCfgChangeAllowed() const;

    Ui::RcClientManagerWidget *ui;

    RcClientConfigMap config;
    PnpInfoCache pnpCache;
    bool hasNewPnpCache = false;
    int emit_client_config_guard = 0;
    FsmState fsmState = FsmState::Unknown;
    ClientEnableTableModel * const enabledModel;
    RcClientStatusModel * const statusModel;
    MergeColumnsTableModel * const mergeModel;
    RcClientTreeModel * const rcClientTreeModel;
    QMap<ClientIndex, ClientInfo> clients;
    QTimer * const applyCacheTimer;
};

#endif // RCCLIENTMANAGERWIDGET_H
