/*
**    Copyright 2010-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef REGIO_MLINKDEVICE_H
#define REGIO_MLINKDEVICE_H

#include <cstdint>
#include <string>
#include <vector>

#include <QElapsedTimer>
#include <QObject>

namespace mlink {

class RegIoPacket;
class RegOp;
class RegIOBase;
class RegOpVector;

class MlinkDevice : public QObject
{
    Q_OBJECT
public:
    MlinkDevice(RegIOBase &p, int address, QObject *parent);

    bool hasAddressExpired() const;
    void setMaxRetryCount(int value) { maxRetryCount = value; }
    bool regOpExec(RegOpVector &v);

    /// read wordCount 32-bit words from RAM starting from startWord offset
    bool memReadBlk(int startWord, size_t wordCount, std::vector<quint32> *result);

    /// write 32-bit words to RAM starting from startWord offset
    bool memWriteBlk(int startWord, const std::vector<quint32> &data);

signals:
    void protocolTimeout();
    void protocolMessage(const QString &);

public slots:

protected:
    QString getIdent() const;

private:
    bool validateIoAck(const RegIoPacket &tx, const RegIoPacket &rx, const std::string &description = std::string());
    bool ctrlExchangeSingle(const RegIoPacket &tx_in, RegIoPacket *rx_out); ///< Send, receive and validate control packet

    /// maximum number of data words for single RAM read/write operation
    size_t getMaxRamRequestSize() const;
    /// maximum number of 16-bit registers for single register read/write operation
    size_t getMaxRegRequestSize() const;
    int nextTxPacketSequenceNumber() { return txPacketSequenceNumber++; }

    RegIOBase &link;
    int myAddress = 0;
    int txPacketSequenceNumber = 0;
    int maxRetryCount = 5;
    QElapsedTimer protocolTimer;
};

} // namespace mlink

#endif /* REGIO_MLINKDEVICE_H */
