/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef PACKETRINGBUFFER_H
#define PACKETRINGBUFFER_H

#include <QByteArray>
#include <QSemaphore>
#include <QVector>

#include "PacketBuffer.h"

class PacketRingBuffer
{
public:
    PacketRingBuffer(int taskNum);

    QSemaphore *getDataSizeSem() { return &dataSize; }
    const PacketBuffer &getPendingTask(uint shift) {
        return tasks[static_cast<int>((taskIndexSend+shift)%taskNum)];
    }
    void releaseDataSem(uint n) {
        shiftIndex(taskIndexSend, n);
        shiftIndex(taskIndexErase, n);
        freeSize.release(static_cast<int>(n));
    }

    inline bool hasFreeTask() { return (taskIndexWrite+1)%taskNum!=taskIndexErase; }
    inline bool hasFreeTask(int n) { return getFreeTask() >= n; }
    inline int getFreeTask() {
        return static_cast<int>((taskNum+taskIndexErase-taskIndexWrite-1)%taskNum);
    }
    inline int getTodoTask() {
        return static_cast<int>((taskIndexWrite+taskNum-taskIndexSend)%taskNum);
    }
    inline int getTaskToErase() {
        return static_cast<int>((taskIndexSend+taskNum-taskIndexErase)%taskNum);
    }
    void shiftIndex(uint &i,uint size=1) { i+=size; i%=taskNum; }

    QByteArray buf;

    QVector<PacketBuffer> tasks;
    const uint taskNum;
    uint taskIndexWrite = 0;
    uint taskIndexSend = 0;
    uint taskIndexErase = 0;

    QSemaphore freeSize;
    QSemaphore dataSize;
};

#endif // PACKETRINGBUFFER_H
