//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef WAVEFORMPLOT_H
#define WAVEFORMPLOT_H

#include "qwt_plot_marker.h"
#include <qxw/qxw_global.h>
#include <qxw/QxwPlot.h>
#include <qxw/WaveformData.h>

class QwtSymbol;

class QXW_EXPORT WaveformPlot: public QxwPlot
{
    Q_OBJECT

public:
    struct MarkerData
    {
        double xvalue = 0;
        double yvalue = 0;
        QwtPlotMarker::LineStyle lineStyle = QwtPlotMarker::HLine;
        Qt::Alignment align = Qt::AlignRight | Qt::AlignBottom;
        QwtText label;
        QPen pen = QPen(QColor(Qt::black), 1, Qt::DotLine);
    };
    WaveformPlot(QWidget *p = 0);
    virtual ~WaveformPlot();

    //! Add or reuse curve associated with channel n
    //    void setData(unsigned int id, const double *px, const double *py, unsigned int len);
    //    void setNumCurves(const unsigned int n);
    //    WFCurve findCurve(int id) {return curves[id];}
    //    void setData(const WaveformData &);
    void setWaveformData(const WaveformDataAll &wda);
    void setAllData(const WaveformDataAll &wda) { setWaveformData(wda); }
    void setMarkers(const std::vector<MarkerData> &v);
    virtual void clear();
    virtual void clearMarkers();
    static QColor standardColor(int index);

private:

    QxwMarkerSet markers;
};

#endif // WAVEFORMPLOT_H
